from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.partition_incremental_time_window_spec import (
        PartitionIncrementalTimeWindowSpec,
    )
    from ..models.physical_table_name import PhysicalTableName
    from ..models.timestamp_columns_spec import TimestampColumnsSpec


T = TypeVar("T", bound="TablePartitioningModel")


@_attrs_define
class TablePartitioningModel:
    """Table model with objects that describe the table partitioning.

    Attributes:
        connection_name (Union[Unset, str]): Connection name.
        target (Union[Unset, PhysicalTableName]):
        timestamp_columns (Union[Unset, TimestampColumnsSpec]):
        incremental_time_window (Union[Unset, PartitionIncrementalTimeWindowSpec]):
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete this object.
    """

    connection_name: Union[Unset, str] = UNSET
    target: Union[Unset, "PhysicalTableName"] = UNSET
    timestamp_columns: Union[Unset, "TimestampColumnsSpec"] = UNSET
    incremental_time_window: Union[Unset, "PartitionIncrementalTimeWindowSpec"] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_name = self.connection_name
        target: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.target, Unset):
            target = self.target.to_dict()

        timestamp_columns: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.timestamp_columns, Unset):
            timestamp_columns = self.timestamp_columns.to_dict()

        incremental_time_window: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.incremental_time_window, Unset):
            incremental_time_window = self.incremental_time_window.to_dict()

        can_edit = self.can_edit

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_name is not UNSET:
            field_dict["connection_name"] = connection_name
        if target is not UNSET:
            field_dict["target"] = target
        if timestamp_columns is not UNSET:
            field_dict["timestamp_columns"] = timestamp_columns
        if incremental_time_window is not UNSET:
            field_dict["incremental_time_window"] = incremental_time_window
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.partition_incremental_time_window_spec import (
            PartitionIncrementalTimeWindowSpec,
        )
        from ..models.physical_table_name import PhysicalTableName
        from ..models.timestamp_columns_spec import TimestampColumnsSpec

        d = src_dict.copy()
        connection_name = d.pop("connection_name", UNSET)

        _target = d.pop("target", UNSET)
        target: Union[Unset, PhysicalTableName]
        if isinstance(_target, Unset):
            target = UNSET
        else:
            target = PhysicalTableName.from_dict(_target)

        _timestamp_columns = d.pop("timestamp_columns", UNSET)
        timestamp_columns: Union[Unset, TimestampColumnsSpec]
        if isinstance(_timestamp_columns, Unset):
            timestamp_columns = UNSET
        else:
            timestamp_columns = TimestampColumnsSpec.from_dict(_timestamp_columns)

        _incremental_time_window = d.pop("incremental_time_window", UNSET)
        incremental_time_window: Union[Unset, PartitionIncrementalTimeWindowSpec]
        if isinstance(_incremental_time_window, Unset):
            incremental_time_window = UNSET
        else:
            incremental_time_window = PartitionIncrementalTimeWindowSpec.from_dict(
                _incremental_time_window
            )

        can_edit = d.pop("can_edit", UNSET)

        table_partitioning_model = cls(
            connection_name=connection_name,
            target=target,
            timestamp_columns=timestamp_columns,
            incremental_time_window=incremental_time_window,
            can_edit=can_edit,
        )

        table_partitioning_model.additional_properties = d
        return table_partitioning_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DetectedDatatypeCategory(str, Enum):
    BOOLEANS = "booleans"
    DATES = "dates"
    DATETIMES = "datetimes"
    FLOATS = "floats"
    INTEGERS = "integers"
    MIXED = "mixed"
    TEXTS = "texts"
    TIMESTAMPS = "timestamps"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="RuleListModel")


@_attrs_define
class RuleListModel:
    """Rule list model

    Attributes:
        rule_name (Union[Unset, str]): Rule name without the folder.
        full_rule_name (Union[Unset, str]): Full rule name, including the folder within the "rules" rule folder.
        custom (Union[Unset, bool]): This rule has is a custom rule or was customized by the user. This is a read-only
            value.
        built_in (Union[Unset, bool]): This rule is provided with DQOps as a built-in rule. This is a read-only value.
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete this object.
        yaml_parsing_error (Union[Unset, str]): Optional parsing error that was captured when parsing the YAML file.
            This field is null when the YAML file is valid. If an error was captured, this field returns the file parsing
            error message and the file location.
    """

    rule_name: Union[Unset, str] = UNSET
    full_rule_name: Union[Unset, str] = UNSET
    custom: Union[Unset, bool] = UNSET
    built_in: Union[Unset, bool] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    yaml_parsing_error: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        rule_name = self.rule_name
        full_rule_name = self.full_rule_name
        custom = self.custom
        built_in = self.built_in
        can_edit = self.can_edit
        yaml_parsing_error = self.yaml_parsing_error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if rule_name is not UNSET:
            field_dict["rule_name"] = rule_name
        if full_rule_name is not UNSET:
            field_dict["full_rule_name"] = full_rule_name
        if custom is not UNSET:
            field_dict["custom"] = custom
        if built_in is not UNSET:
            field_dict["built_in"] = built_in
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if yaml_parsing_error is not UNSET:
            field_dict["yaml_parsing_error"] = yaml_parsing_error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        rule_name = d.pop("rule_name", UNSET)

        full_rule_name = d.pop("full_rule_name", UNSET)

        custom = d.pop("custom", UNSET)

        built_in = d.pop("built_in", UNSET)

        can_edit = d.pop("can_edit", UNSET)

        yaml_parsing_error = d.pop("yaml_parsing_error", UNSET)

        rule_list_model = cls(
            rule_name=rule_name,
            full_rule_name=full_rule_name,
            custom=custom,
            built_in=built_in,
            can_edit=can_edit,
            yaml_parsing_error=yaml_parsing_error,
        )

        rule_list_model.additional_properties = d
        return rule_list_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ColumnNumericNumberBelowMinValueCountSensorParametersSpec")


@_attrs_define
class ColumnNumericNumberBelowMinValueCountSensorParametersSpec:
    """
    Attributes:
        filter_ (Union[Unset, str]): SQL WHERE clause added to the sensor query. Both the table level filter and a
            sensor query filter are added, separated by an AND operator.
        min_value (Union[Unset, float]): This field can be used to define custom value. In order to define custom value,
            user should write correct value as an integer. If value is not defined by user then default value is 0
    """

    filter_: Union[Unset, str] = UNSET
    min_value: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        filter_ = self.filter_
        min_value = self.min_value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if filter_ is not UNSET:
            field_dict["filter"] = filter_
        if min_value is not UNSET:
            field_dict["min_value"] = min_value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        filter_ = d.pop("filter", UNSET)

        min_value = d.pop("min_value", UNSET)

        column_numeric_number_below_min_value_count_sensor_parameters_spec = cls(
            filter_=filter_,
            min_value=min_value,
        )

        column_numeric_number_below_min_value_count_sensor_parameters_spec.additional_properties = (
            d
        )
        return column_numeric_number_below_min_value_count_sensor_parameters_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.compression_type import CompressionType
from ..models.new_line_character_type import NewLineCharacterType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CsvFileFormatSpec")


@_attrs_define
class CsvFileFormatSpec:
    """
    Attributes:
        all_varchar (Union[Unset, bool]): Option to skip type detection for CSV parsing and assume all columns to be of
            type VARCHAR.
        allow_quoted_nulls (Union[Unset, bool]): Option to allow the conversion of quoted values to NULL values.
        auto_detect (Union[Unset, bool]): Enables auto detection of CSV parameters.
        compression (Union[Unset, CompressionType]):
        no_compression_extension (Union[Unset, bool]): Whether the compression extension is present at the end of the
            file name.
        file_extension (Union[Unset, str]): Specifies a custom file name extension. The default is .csv.
        dateformat (Union[Unset, str]): Specifies the date format to use when parsing dates.
        decimal_separator (Union[Unset, str]): The decimal separator of numbers.
        delim (Union[Unset, str]): Specifies the string that separates columns within each row (line) of the file.
        escape (Union[Unset, str]): Specifies the string that should appear before a data character sequence that
            matches the quote value.
        filename (Union[Unset, bool]): Whether or not an extra filename column should be included in the result.
        header (Union[Unset, bool]): Specifies that the file contains a header line with the names of each column in the
            file.
        hive_partitioning (Union[Unset, bool]): Whether or not to interpret the path as a hive partitioned path.
        ignore_errors (Union[Unset, bool]): Option to ignore any parsing errors encountered - and instead ignore rows
            with errors.
        new_line (Union[Unset, NewLineCharacterType]):
        quote (Union[Unset, str]): Specifies the quoting string to be used when a data value is quoted.
        sample_size (Union[Unset, int]): The number of sample rows for auto detection of parameters.
        skip (Union[Unset, int]): The number of lines at the top of the file to skip.
        timestampformat (Union[Unset, str]): Specifies the date format to use when parsing timestamps.
    """

    all_varchar: Union[Unset, bool] = UNSET
    allow_quoted_nulls: Union[Unset, bool] = UNSET
    auto_detect: Union[Unset, bool] = UNSET
    compression: Union[Unset, CompressionType] = UNSET
    no_compression_extension: Union[Unset, bool] = UNSET
    file_extension: Union[Unset, str] = UNSET
    dateformat: Union[Unset, str] = UNSET
    decimal_separator: Union[Unset, str] = UNSET
    delim: Union[Unset, str] = UNSET
    escape: Union[Unset, str] = UNSET
    filename: Union[Unset, bool] = UNSET
    header: Union[Unset, bool] = UNSET
    hive_partitioning: Union[Unset, bool] = UNSET
    ignore_errors: Union[Unset, bool] = UNSET
    new_line: Union[Unset, NewLineCharacterType] = UNSET
    quote: Union[Unset, str] = UNSET
    sample_size: Union[Unset, int] = UNSET
    skip: Union[Unset, int] = UNSET
    timestampformat: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        all_varchar = self.all_varchar
        allow_quoted_nulls = self.allow_quoted_nulls
        auto_detect = self.auto_detect
        compression: Union[Unset, str] = UNSET
        if not isinstance(self.compression, Unset):
            compression = self.compression.value

        no_compression_extension = self.no_compression_extension
        file_extension = self.file_extension
        dateformat = self.dateformat
        decimal_separator = self.decimal_separator
        delim = self.delim
        escape = self.escape
        filename = self.filename
        header = self.header
        hive_partitioning = self.hive_partitioning
        ignore_errors = self.ignore_errors
        new_line: Union[Unset, str] = UNSET
        if not isinstance(self.new_line, Unset):
            new_line = self.new_line.value

        quote = self.quote
        sample_size = self.sample_size
        skip = self.skip
        timestampformat = self.timestampformat

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if all_varchar is not UNSET:
            field_dict["all_varchar"] = all_varchar
        if allow_quoted_nulls is not UNSET:
            field_dict["allow_quoted_nulls"] = allow_quoted_nulls
        if auto_detect is not UNSET:
            field_dict["auto_detect"] = auto_detect
        if compression is not UNSET:
            field_dict["compression"] = compression
        if no_compression_extension is not UNSET:
            field_dict["no_compression_extension"] = no_compression_extension
        if file_extension is not UNSET:
            field_dict["file_extension"] = file_extension
        if dateformat is not UNSET:
            field_dict["dateformat"] = dateformat
        if decimal_separator is not UNSET:
            field_dict["decimal_separator"] = decimal_separator
        if delim is not UNSET:
            field_dict["delim"] = delim
        if escape is not UNSET:
            field_dict["escape"] = escape
        if filename is not UNSET:
            field_dict["filename"] = filename
        if header is not UNSET:
            field_dict["header"] = header
        if hive_partitioning is not UNSET:
            field_dict["hive_partitioning"] = hive_partitioning
        if ignore_errors is not UNSET:
            field_dict["ignore_errors"] = ignore_errors
        if new_line is not UNSET:
            field_dict["new_line"] = new_line
        if quote is not UNSET:
            field_dict["quote"] = quote
        if sample_size is not UNSET:
            field_dict["sample_size"] = sample_size
        if skip is not UNSET:
            field_dict["skip"] = skip
        if timestampformat is not UNSET:
            field_dict["timestampformat"] = timestampformat

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        all_varchar = d.pop("all_varchar", UNSET)

        allow_quoted_nulls = d.pop("allow_quoted_nulls", UNSET)

        auto_detect = d.pop("auto_detect", UNSET)

        _compression = d.pop("compression", UNSET)
        compression: Union[Unset, CompressionType]
        if isinstance(_compression, Unset):
            compression = UNSET
        else:
            compression = CompressionType(_compression)

        no_compression_extension = d.pop("no_compression_extension", UNSET)

        file_extension = d.pop("file_extension", UNSET)

        dateformat = d.pop("dateformat", UNSET)

        decimal_separator = d.pop("decimal_separator", UNSET)

        delim = d.pop("delim", UNSET)

        escape = d.pop("escape", UNSET)

        filename = d.pop("filename", UNSET)

        header = d.pop("header", UNSET)

        hive_partitioning = d.pop("hive_partitioning", UNSET)

        ignore_errors = d.pop("ignore_errors", UNSET)

        _new_line = d.pop("new_line", UNSET)
        new_line: Union[Unset, NewLineCharacterType]
        if isinstance(_new_line, Unset):
            new_line = UNSET
        else:
            new_line = NewLineCharacterType(_new_line)

        quote = d.pop("quote", UNSET)

        sample_size = d.pop("sample_size", UNSET)

        skip = d.pop("skip", UNSET)

        timestampformat = d.pop("timestampformat", UNSET)

        csv_file_format_spec = cls(
            all_varchar=all_varchar,
            allow_quoted_nulls=allow_quoted_nulls,
            auto_detect=auto_detect,
            compression=compression,
            no_compression_extension=no_compression_extension,
            file_extension=file_extension,
            dateformat=dateformat,
            decimal_separator=decimal_separator,
            delim=delim,
            escape=escape,
            filename=filename,
            header=header,
            hive_partitioning=hive_partitioning,
            ignore_errors=ignore_errors,
            new_line=new_line,
            quote=quote,
            sample_size=sample_size,
            skip=skip,
            timestampformat=timestampformat,
        )

        csv_file_format_spec.additional_properties = d
        return csv_file_format_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DqoJobStatus(str, Enum):
    CANCELLED = "cancelled"
    CANCEL_REQUESTED = "cancel_requested"
    FAILED = "failed"
    FINISHED = "finished"
    QUEUED = "queued"
    RUNNING = "running"
    WAITING = "waiting"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.duration import Duration


T = TypeVar("T", bound="TemporalUnit")


@_attrs_define
class TemporalUnit:
    """
    Attributes:
        date_based (Union[Unset, bool]):
        time_based (Union[Unset, bool]):
        duration (Union[Unset, Duration]):
        duration_estimated (Union[Unset, bool]):
    """

    date_based: Union[Unset, bool] = UNSET
    time_based: Union[Unset, bool] = UNSET
    duration: Union[Unset, "Duration"] = UNSET
    duration_estimated: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        date_based = self.date_based
        time_based = self.time_based
        duration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.duration, Unset):
            duration = self.duration.to_dict()

        duration_estimated = self.duration_estimated

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if date_based is not UNSET:
            field_dict["dateBased"] = date_based
        if time_based is not UNSET:
            field_dict["timeBased"] = time_based
        if duration is not UNSET:
            field_dict["duration"] = duration
        if duration_estimated is not UNSET:
            field_dict["durationEstimated"] = duration_estimated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.duration import Duration

        d = src_dict.copy()
        date_based = d.pop("dateBased", UNSET)

        time_based = d.pop("timeBased", UNSET)

        _duration = d.pop("duration", UNSET)
        duration: Union[Unset, Duration]
        if isinstance(_duration, Unset):
            duration = UNSET
        else:
            duration = Duration.from_dict(_duration)

        duration_estimated = d.pop("durationEstimated", UNSET)

        temporal_unit = cls(
            date_based=date_based,
            time_based=time_based,
            duration=duration,
            duration_estimated=duration_estimated,
        )

        temporal_unit.additional_properties = d
        return temporal_unit

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DefaultRuleSeverityLevel(str, Enum):
    ERROR = "error"
    FATAL = "fatal"
    NONE = "none"
    WARNING = "warning"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class SqlServerAuthenticationMode(str, Enum):
    ACTIVE_DIRECTORY_DEFAULT = "active_directory_default"
    ACTIVE_DIRECTORY_PASSWORD = "active_directory_password"
    ACTIVE_DIRECTORY_SERVICE_PRINCIPAL = "active_directory_service_principal"
    SQL_PASSWORD = "sql_password"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.check_model import CheckModel
    from ..models.check_search_filters import CheckSearchFilters
    from ..models.delete_stored_data_queue_job_parameters import (
        DeleteStoredDataQueueJobParameters,
    )


T = TypeVar("T", bound="QualityCategoryModel")


@_attrs_define
class QualityCategoryModel:
    """Model that returns the form definition and the form data to edit all checks within a single category.

    Attributes:
        category (Union[Unset, str]): Data quality check category name.
        comparison_name (Union[Unset, str]): The name of the reference table configuration used for a cross table data
            comparison (when the category is 'comparisons').
        compare_to_column (Union[Unset, str]): The name of the column in the reference table that is compared.
        help_text (Union[Unset, str]): Help text that describes the category.
        checks (Union[Unset, List['CheckModel']]): List of data quality checks within the category.
        run_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter, identifies which
            checks on which tables and columns should be executed.
        data_clean_job_template (Union[Unset, DeleteStoredDataQueueJobParameters]):
    """

    category: Union[Unset, str] = UNSET
    comparison_name: Union[Unset, str] = UNSET
    compare_to_column: Union[Unset, str] = UNSET
    help_text: Union[Unset, str] = UNSET
    checks: Union[Unset, List["CheckModel"]] = UNSET
    run_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    data_clean_job_template: Union[Unset, "DeleteStoredDataQueueJobParameters"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        category = self.category
        comparison_name = self.comparison_name
        compare_to_column = self.compare_to_column
        help_text = self.help_text
        checks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.checks, Unset):
            checks = []
            for checks_item_data in self.checks:
                checks_item = checks_item_data.to_dict()

                checks.append(checks_item)

        run_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_checks_job_template, Unset):
            run_checks_job_template = self.run_checks_job_template.to_dict()

        data_clean_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_clean_job_template, Unset):
            data_clean_job_template = self.data_clean_job_template.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if category is not UNSET:
            field_dict["category"] = category
        if comparison_name is not UNSET:
            field_dict["comparison_name"] = comparison_name
        if compare_to_column is not UNSET:
            field_dict["compare_to_column"] = compare_to_column
        if help_text is not UNSET:
            field_dict["help_text"] = help_text
        if checks is not UNSET:
            field_dict["checks"] = checks
        if run_checks_job_template is not UNSET:
            field_dict["run_checks_job_template"] = run_checks_job_template
        if data_clean_job_template is not UNSET:
            field_dict["data_clean_job_template"] = data_clean_job_template

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.check_model import CheckModel
        from ..models.check_search_filters import CheckSearchFilters
        from ..models.delete_stored_data_queue_job_parameters import (
            DeleteStoredDataQueueJobParameters,
        )

        d = src_dict.copy()
        category = d.pop("category", UNSET)

        comparison_name = d.pop("comparison_name", UNSET)

        compare_to_column = d.pop("compare_to_column", UNSET)

        help_text = d.pop("help_text", UNSET)

        checks = []
        _checks = d.pop("checks", UNSET)
        for checks_item_data in _checks or []:
            checks_item = CheckModel.from_dict(checks_item_data)

            checks.append(checks_item)

        _run_checks_job_template = d.pop("run_checks_job_template", UNSET)
        run_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_checks_job_template, Unset):
            run_checks_job_template = UNSET
        else:
            run_checks_job_template = CheckSearchFilters.from_dict(
                _run_checks_job_template
            )

        _data_clean_job_template = d.pop("data_clean_job_template", UNSET)
        data_clean_job_template: Union[Unset, DeleteStoredDataQueueJobParameters]
        if isinstance(_data_clean_job_template, Unset):
            data_clean_job_template = UNSET
        else:
            data_clean_job_template = DeleteStoredDataQueueJobParameters.from_dict(
                _data_clean_job_template
            )

        quality_category_model = cls(
            category=category,
            comparison_name=comparison_name,
            compare_to_column=compare_to_column,
            help_text=help_text,
            checks=checks,
            run_checks_job_template=run_checks_job_template,
            data_clean_job_template=data_clean_job_template,
        )

        quality_category_model.additional_properties = d
        return quality_category_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.rule_severity_level import RuleSeverityLevel
from ..types import UNSET, Unset

T = TypeVar("T", bound="DimensionCurrentDataQualityStatusModel")


@_attrs_define
class DimensionCurrentDataQualityStatusModel:
    """The summary of the current data quality status for one data quality dimension

    Attributes:
        dimension (Union[Unset, str]): Data quality dimension name. The most popular dimensions are: Completeness,
            Uniqueness, Timeliness, Validity, Consistency, Accuracy, Availability.
        current_severity (Union[Unset, RuleSeverityLevel]):
        highest_historical_severity (Union[Unset, RuleSeverityLevel]):
        last_check_executed_at (Union[Unset, int]): The UTC timestamp when the most recent data quality check was
            executed on the table.
        executed_checks (Union[Unset, int]): The total number of most recent checks that were executed on the table for
            one data quality dimension. Table comparison checks that are comparing groups of data are counted as the number
            of compared data groups.
        valid_results (Union[Unset, int]): The number of most recent valid data quality checks that passed without
            raising any issues.
        warnings (Union[Unset, int]): The number of most recent data quality checks that failed by raising a warning
            severity data quality issue.
        errors (Union[Unset, int]): The number of most recent data quality checks that failed by raising an error
            severity data quality issue.
        fatals (Union[Unset, int]): The number of most recent data quality checks that failed by raising a fatal
            severity data quality issue.
        execution_errors (Union[Unset, int]): The number of data quality check execution errors that were reported due
            to access issues to the data source, invalid mapping in DQOps, invalid queries in data quality sensors or
            invalid python rules. When an execution error is reported, the configuration of a data quality check on a table
            must be updated.
        data_quality_kpi (Union[Unset, float]): Data quality KPI score for the data quality dimension, measured as a
            percentage of passed data quality checks. DQOps counts data quality issues at a warning severity level as passed
            checks. The data quality KPI score is a value in the range 0..100.
    """

    dimension: Union[Unset, str] = UNSET
    current_severity: Union[Unset, RuleSeverityLevel] = UNSET
    highest_historical_severity: Union[Unset, RuleSeverityLevel] = UNSET
    last_check_executed_at: Union[Unset, int] = UNSET
    executed_checks: Union[Unset, int] = UNSET
    valid_results: Union[Unset, int] = UNSET
    warnings: Union[Unset, int] = UNSET
    errors: Union[Unset, int] = UNSET
    fatals: Union[Unset, int] = UNSET
    execution_errors: Union[Unset, int] = UNSET
    data_quality_kpi: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        dimension = self.dimension
        current_severity: Union[Unset, str] = UNSET
        if not isinstance(self.current_severity, Unset):
            current_severity = self.current_severity.value

        highest_historical_severity: Union[Unset, str] = UNSET
        if not isinstance(self.highest_historical_severity, Unset):
            highest_historical_severity = self.highest_historical_severity.value

        last_check_executed_at = self.last_check_executed_at
        executed_checks = self.executed_checks
        valid_results = self.valid_results
        warnings = self.warnings
        errors = self.errors
        fatals = self.fatals
        execution_errors = self.execution_errors
        data_quality_kpi = self.data_quality_kpi

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if dimension is not UNSET:
            field_dict["dimension"] = dimension
        if current_severity is not UNSET:
            field_dict["current_severity"] = current_severity
        if highest_historical_severity is not UNSET:
            field_dict["highest_historical_severity"] = highest_historical_severity
        if last_check_executed_at is not UNSET:
            field_dict["last_check_executed_at"] = last_check_executed_at
        if executed_checks is not UNSET:
            field_dict["executed_checks"] = executed_checks
        if valid_results is not UNSET:
            field_dict["valid_results"] = valid_results
        if warnings is not UNSET:
            field_dict["warnings"] = warnings
        if errors is not UNSET:
            field_dict["errors"] = errors
        if fatals is not UNSET:
            field_dict["fatals"] = fatals
        if execution_errors is not UNSET:
            field_dict["execution_errors"] = execution_errors
        if data_quality_kpi is not UNSET:
            field_dict["data_quality_kpi"] = data_quality_kpi

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        dimension = d.pop("dimension", UNSET)

        _current_severity = d.pop("current_severity", UNSET)
        current_severity: Union[Unset, RuleSeverityLevel]
        if isinstance(_current_severity, Unset):
            current_severity = UNSET
        else:
            current_severity = RuleSeverityLevel(_current_severity)

        _highest_historical_severity = d.pop("highest_historical_severity", UNSET)
        highest_historical_severity: Union[Unset, RuleSeverityLevel]
        if isinstance(_highest_historical_severity, Unset):
            highest_historical_severity = UNSET
        else:
            highest_historical_severity = RuleSeverityLevel(
                _highest_historical_severity
            )

        last_check_executed_at = d.pop("last_check_executed_at", UNSET)

        executed_checks = d.pop("executed_checks", UNSET)

        valid_results = d.pop("valid_results", UNSET)

        warnings = d.pop("warnings", UNSET)

        errors = d.pop("errors", UNSET)

        fatals = d.pop("fatals", UNSET)

        execution_errors = d.pop("execution_errors", UNSET)

        data_quality_kpi = d.pop("data_quality_kpi", UNSET)

        dimension_current_data_quality_status_model = cls(
            dimension=dimension,
            current_severity=current_severity,
            highest_historical_severity=highest_historical_severity,
            last_check_executed_at=last_check_executed_at,
            executed_checks=executed_checks,
            valid_results=valid_results,
            warnings=warnings,
            errors=errors,
            fatals=fatals,
            execution_errors=execution_errors,
            data_quality_kpi=data_quality_kpi,
        )

        dimension_current_data_quality_status_model.additional_properties = d
        return dimension_current_data_quality_status_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_uniqueness_duplicate_percent_sensor_parameters_spec import (
        ColumnUniquenessDuplicatePercentSensorParametersSpec,
    )


T = TypeVar("T", bound="ColumnUniquenessDuplicatePercentStatisticsCollectorSpec")


@_attrs_define
class ColumnUniquenessDuplicatePercentStatisticsCollectorSpec:
    """
    Attributes:
        disabled (Union[Unset, bool]): Disables this profiler. Only enabled profilers are executed during a profiling
            process.
        parameters (Union[Unset, ColumnUniquenessDuplicatePercentSensorParametersSpec]):
    """

    disabled: Union[Unset, bool] = UNSET
    parameters: Union[Unset, "ColumnUniquenessDuplicatePercentSensorParametersSpec"] = (
        UNSET
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        disabled = self.disabled
        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if parameters is not UNSET:
            field_dict["parameters"] = parameters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_uniqueness_duplicate_percent_sensor_parameters_spec import (
            ColumnUniquenessDuplicatePercentSensorParametersSpec,
        )

        d = src_dict.copy()
        disabled = d.pop("disabled", UNSET)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, ColumnUniquenessDuplicatePercentSensorParametersSpec]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = ColumnUniquenessDuplicatePercentSensorParametersSpec.from_dict(
                _parameters
            )

        column_uniqueness_duplicate_percent_statistics_collector_spec = cls(
            disabled=disabled,
            parameters=parameters,
        )

        column_uniqueness_duplicate_percent_statistics_collector_spec.additional_properties = (
            d
        )
        return column_uniqueness_duplicate_percent_statistics_collector_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.check_type import CheckType
from ..models.error_sample_result_data_type import ErrorSampleResultDataType
from ..models.time_period_gradient import TimePeriodGradient
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.error_sample_entry_model_result import ErrorSampleEntryModelResult


T = TypeVar("T", bound="ErrorSampleEntryModel")


@_attrs_define
class ErrorSampleEntryModel:
    """
    Attributes:
        id (Union[Unset, str]): Unique ID of the error sample value
        result (Union[Unset, ErrorSampleEntryModelResult]): Error sample (value)
        result_data_type (Union[Unset, ErrorSampleResultDataType]):
        collected_at (Union[Unset, datetime.datetime]): The local timestamp when the error sample was collected
        sample_index (Union[Unset, int]): The index of the result that was returned. Identifies a single error sample
            within a list.
        check_type (Union[Unset, CheckType]):
        time_gradient (Union[Unset, TimePeriodGradient]):
        duration_ms (Union[Unset, int]): Execution duration (ms)
        executed_at (Union[Unset, int]): Executed at
        row_id_1 (Union[Unset, str]): The value of the first column of the unique identifier that identifies a row
            containing the error sample.
        row_id_2 (Union[Unset, str]): The value of the second column of the unique identifier that identifies a row
            containing the error sample.
        row_id_3 (Union[Unset, str]): The value of the third column of the unique identifier that identifies a row
            containing the error sample.
        row_id_4 (Union[Unset, str]): The value of the fourth column of the unique identifier that identifies a row
            containing the error sample.
        row_id_5 (Union[Unset, str]): The value of the fifth column of the unique identifier that identifies a row
            containing the error sample.
        column_name (Union[Unset, str]): Column name
        data_group (Union[Unset, str]): Data group
        provider (Union[Unset, str]): Provider name
        quality_dimension (Union[Unset, str]): Data quality dimension
        sensor_name (Union[Unset, str]): Sensor name
        table_comparison (Union[Unset, str]): Table comparison name
    """

    id: Union[Unset, str] = UNSET
    result: Union[Unset, "ErrorSampleEntryModelResult"] = UNSET
    result_data_type: Union[Unset, ErrorSampleResultDataType] = UNSET
    collected_at: Union[Unset, datetime.datetime] = UNSET
    sample_index: Union[Unset, int] = UNSET
    check_type: Union[Unset, CheckType] = UNSET
    time_gradient: Union[Unset, TimePeriodGradient] = UNSET
    duration_ms: Union[Unset, int] = UNSET
    executed_at: Union[Unset, int] = UNSET
    row_id_1: Union[Unset, str] = UNSET
    row_id_2: Union[Unset, str] = UNSET
    row_id_3: Union[Unset, str] = UNSET
    row_id_4: Union[Unset, str] = UNSET
    row_id_5: Union[Unset, str] = UNSET
    column_name: Union[Unset, str] = UNSET
    data_group: Union[Unset, str] = UNSET
    provider: Union[Unset, str] = UNSET
    quality_dimension: Union[Unset, str] = UNSET
    sensor_name: Union[Unset, str] = UNSET
    table_comparison: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        result: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.result, Unset):
            result = self.result.to_dict()

        result_data_type: Union[Unset, str] = UNSET
        if not isinstance(self.result_data_type, Unset):
            result_data_type = self.result_data_type.value

        collected_at: Union[Unset, str] = UNSET
        if not isinstance(self.collected_at, Unset):
            collected_at = self.collected_at.isoformat()

        sample_index = self.sample_index
        check_type: Union[Unset, str] = UNSET
        if not isinstance(self.check_type, Unset):
            check_type = self.check_type.value

        time_gradient: Union[Unset, str] = UNSET
        if not isinstance(self.time_gradient, Unset):
            time_gradient = self.time_gradient.value

        duration_ms = self.duration_ms
        executed_at = self.executed_at
        row_id_1 = self.row_id_1
        row_id_2 = self.row_id_2
        row_id_3 = self.row_id_3
        row_id_4 = self.row_id_4
        row_id_5 = self.row_id_5
        column_name = self.column_name
        data_group = self.data_group
        provider = self.provider
        quality_dimension = self.quality_dimension
        sensor_name = self.sensor_name
        table_comparison = self.table_comparison

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if result is not UNSET:
            field_dict["result"] = result
        if result_data_type is not UNSET:
            field_dict["resultDataType"] = result_data_type
        if collected_at is not UNSET:
            field_dict["collectedAt"] = collected_at
        if sample_index is not UNSET:
            field_dict["sampleIndex"] = sample_index
        if check_type is not UNSET:
            field_dict["checkType"] = check_type
        if time_gradient is not UNSET:
            field_dict["timeGradient"] = time_gradient
        if duration_ms is not UNSET:
            field_dict["durationMs"] = duration_ms
        if executed_at is not UNSET:
            field_dict["executedAt"] = executed_at
        if row_id_1 is not UNSET:
            field_dict["rowId1"] = row_id_1
        if row_id_2 is not UNSET:
            field_dict["rowId2"] = row_id_2
        if row_id_3 is not UNSET:
            field_dict["rowId3"] = row_id_3
        if row_id_4 is not UNSET:
            field_dict["rowId4"] = row_id_4
        if row_id_5 is not UNSET:
            field_dict["rowId5"] = row_id_5
        if column_name is not UNSET:
            field_dict["columnName"] = column_name
        if data_group is not UNSET:
            field_dict["dataGroup"] = data_group
        if provider is not UNSET:
            field_dict["provider"] = provider
        if quality_dimension is not UNSET:
            field_dict["qualityDimension"] = quality_dimension
        if sensor_name is not UNSET:
            field_dict["sensorName"] = sensor_name
        if table_comparison is not UNSET:
            field_dict["tableComparison"] = table_comparison

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.error_sample_entry_model_result import ErrorSampleEntryModelResult

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _result = d.pop("result", UNSET)
        result: Union[Unset, ErrorSampleEntryModelResult]
        if isinstance(_result, Unset):
            result = UNSET
        else:
            result = ErrorSampleEntryModelResult.from_dict(_result)

        _result_data_type = d.pop("resultDataType", UNSET)
        result_data_type: Union[Unset, ErrorSampleResultDataType]
        if isinstance(_result_data_type, Unset):
            result_data_type = UNSET
        else:
            result_data_type = ErrorSampleResultDataType(_result_data_type)

        _collected_at = d.pop("collectedAt", UNSET)
        collected_at: Union[Unset, datetime.datetime]
        if isinstance(_collected_at, Unset):
            collected_at = UNSET
        else:
            collected_at = isoparse(_collected_at)

        sample_index = d.pop("sampleIndex", UNSET)

        _check_type = d.pop("checkType", UNSET)
        check_type: Union[Unset, CheckType]
        if isinstance(_check_type, Unset):
            check_type = UNSET
        else:
            check_type = CheckType(_check_type)

        _time_gradient = d.pop("timeGradient", UNSET)
        time_gradient: Union[Unset, TimePeriodGradient]
        if isinstance(_time_gradient, Unset):
            time_gradient = UNSET
        else:
            time_gradient = TimePeriodGradient(_time_gradient)

        duration_ms = d.pop("durationMs", UNSET)

        executed_at = d.pop("executedAt", UNSET)

        row_id_1 = d.pop("rowId1", UNSET)

        row_id_2 = d.pop("rowId2", UNSET)

        row_id_3 = d.pop("rowId3", UNSET)

        row_id_4 = d.pop("rowId4", UNSET)

        row_id_5 = d.pop("rowId5", UNSET)

        column_name = d.pop("columnName", UNSET)

        data_group = d.pop("dataGroup", UNSET)

        provider = d.pop("provider", UNSET)

        quality_dimension = d.pop("qualityDimension", UNSET)

        sensor_name = d.pop("sensorName", UNSET)

        table_comparison = d.pop("tableComparison", UNSET)

        error_sample_entry_model = cls(
            id=id,
            result=result,
            result_data_type=result_data_type,
            collected_at=collected_at,
            sample_index=sample_index,
            check_type=check_type,
            time_gradient=time_gradient,
            duration_ms=duration_ms,
            executed_at=executed_at,
            row_id_1=row_id_1,
            row_id_2=row_id_2,
            row_id_3=row_id_3,
            row_id_4=row_id_4,
            row_id_5=row_id_5,
            column_name=column_name,
            data_group=data_group,
            provider=provider,
            quality_dimension=quality_dimension,
            sensor_name=sensor_name,
            table_comparison=table_comparison,
        )

        error_sample_entry_model.additional_properties = d
        return error_sample_entry_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CommonColumnModel")


@_attrs_define
class CommonColumnModel:
    """Common column model that describes a column name that is frequently used in tables within a connection

    Attributes:
        column_name (Union[Unset, str]): Column name.
        tables_count (Union[Unset, int]): Count of tables that are have a column with this name.
    """

    column_name: Union[Unset, str] = UNSET
    tables_count: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        column_name = self.column_name
        tables_count = self.tables_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if column_name is not UNSET:
            field_dict["column_name"] = column_name
        if tables_count is not UNSET:
            field_dict["tables_count"] = tables_count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        column_name = d.pop("column_name", UNSET)

        tables_count = d.pop("tables_count", UNSET)

        common_column_model = cls(
            column_name=column_name,
            tables_count=tables_count,
        )

        common_column_model.additional_properties = d
        return common_column_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="CommentSpec")


@_attrs_define
class CommentSpec:
    """
    Attributes:
        date (Union[Unset, datetime.datetime]): Comment date and time
        comment_by (Union[Unset, str]): Commented by
        comment (Union[Unset, str]): Comment text
    """

    date: Union[Unset, datetime.datetime] = UNSET
    comment_by: Union[Unset, str] = UNSET
    comment: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        comment_by = self.comment_by
        comment = self.comment

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if comment_by is not UNSET:
            field_dict["comment_by"] = comment_by
        if comment is not UNSET:
            field_dict["comment"] = comment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.datetime]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date)

        comment_by = d.pop("comment_by", UNSET)

        comment = d.pop("comment", UNSET)

        comment_spec = cls(
            date=date,
            comment_by=comment_by,
            comment=comment,
        )

        comment_spec.additional_properties = d
        return comment_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.table_schema_statistics_collectors_spec import (
        TableSchemaStatisticsCollectorsSpec,
    )
    from ..models.table_volume_statistics_collectors_spec import (
        TableVolumeStatisticsCollectorsSpec,
    )


T = TypeVar("T", bound="TableStatisticsCollectorsRootCategoriesSpec")


@_attrs_define
class TableStatisticsCollectorsRootCategoriesSpec:
    """
    Attributes:
        volume (Union[Unset, TableVolumeStatisticsCollectorsSpec]):
        schema (Union[Unset, TableSchemaStatisticsCollectorsSpec]):
    """

    volume: Union[Unset, "TableVolumeStatisticsCollectorsSpec"] = UNSET
    schema: Union[Unset, "TableSchemaStatisticsCollectorsSpec"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        volume: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.volume, Unset):
            volume = self.volume.to_dict()

        schema: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.schema, Unset):
            schema = self.schema.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if volume is not UNSET:
            field_dict["volume"] = volume
        if schema is not UNSET:
            field_dict["schema"] = schema

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.table_schema_statistics_collectors_spec import (
            TableSchemaStatisticsCollectorsSpec,
        )
        from ..models.table_volume_statistics_collectors_spec import (
            TableVolumeStatisticsCollectorsSpec,
        )

        d = src_dict.copy()
        _volume = d.pop("volume", UNSET)
        volume: Union[Unset, TableVolumeStatisticsCollectorsSpec]
        if isinstance(_volume, Unset):
            volume = UNSET
        else:
            volume = TableVolumeStatisticsCollectorsSpec.from_dict(_volume)

        _schema = d.pop("schema", UNSET)
        schema: Union[Unset, TableSchemaStatisticsCollectorsSpec]
        if isinstance(_schema, Unset):
            schema = UNSET
        else:
            schema = TableSchemaStatisticsCollectorsSpec.from_dict(_schema)

        table_statistics_collectors_root_categories_spec = cls(
            volume=volume,
            schema=schema,
        )

        table_statistics_collectors_root_categories_spec.additional_properties = d
        return table_statistics_collectors_root_categories_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.detected_datatype_category import DetectedDatatypeCategory
from ..types import UNSET, Unset

T = TypeVar("T", bound="DetectedDatatypeEqualsRuleParametersSpec")


@_attrs_define
class DetectedDatatypeEqualsRuleParametersSpec:
    """
    Attributes:
        expected_datatype (Union[Unset, DetectedDatatypeCategory]):
    """

    expected_datatype: Union[Unset, DetectedDatatypeCategory] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        expected_datatype: Union[Unset, str] = UNSET
        if not isinstance(self.expected_datatype, Unset):
            expected_datatype = self.expected_datatype.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if expected_datatype is not UNSET:
            field_dict["expected_datatype"] = expected_datatype

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _expected_datatype = d.pop("expected_datatype", UNSET)
        expected_datatype: Union[Unset, DetectedDatatypeCategory]
        if isinstance(_expected_datatype, Unset):
            expected_datatype = UNSET
        else:
            expected_datatype = DetectedDatatypeCategory(_expected_datatype)

        detected_datatype_equals_rule_parameters_spec = cls(
            expected_datatype=expected_datatype,
        )

        detected_datatype_equals_rule_parameters_spec.additional_properties = d
        return detected_datatype_equals_rule_parameters_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.provider_type import ProviderType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.big_query_parameters_spec import BigQueryParametersSpec
    from ..models.check_search_filters import CheckSearchFilters
    from ..models.click_house_parameters_spec import ClickHouseParametersSpec
    from ..models.connection_model_advanced_properties import (
        ConnectionModelAdvancedProperties,
    )
    from ..models.databricks_parameters_spec import DatabricksParametersSpec
    from ..models.db_2_parameters_spec import Db2ParametersSpec
    from ..models.delete_stored_data_queue_job_parameters import (
        DeleteStoredDataQueueJobParameters,
    )
    from ..models.duckdb_parameters_spec import DuckdbParametersSpec
    from ..models.hana_parameters_spec import HanaParametersSpec
    from ..models.maria_db_parameters_spec import MariaDbParametersSpec
    from ..models.mysql_parameters_spec import MysqlParametersSpec
    from ..models.oracle_parameters_spec import OracleParametersSpec
    from ..models.postgresql_parameters_spec import PostgresqlParametersSpec
    from ..models.presto_parameters_spec import PrestoParametersSpec
    from ..models.quest_db_parameters_spec import QuestDbParametersSpec
    from ..models.redshift_parameters_spec import RedshiftParametersSpec
    from ..models.snowflake_parameters_spec import SnowflakeParametersSpec
    from ..models.spark_parameters_spec import SparkParametersSpec
    from ..models.sql_server_parameters_spec import SqlServerParametersSpec
    from ..models.statistics_collector_search_filters import (
        StatisticsCollectorSearchFilters,
    )
    from ..models.teradata_parameters_spec import TeradataParametersSpec
    from ..models.trino_parameters_spec import TrinoParametersSpec


T = TypeVar("T", bound="ConnectionModel")


@_attrs_define
class ConnectionModel:
    """Connection model with a subset of parameters, excluding all nested objects.

    Attributes:
        connection_name (Union[Unset, str]): Connection name.
        connection_hash (Union[Unset, int]): Connection hash that identifies the connection using a unique hash code.
        parallel_jobs_limit (Union[Unset, int]): The concurrency limit for the maximum number of parallel SQL queries
            executed on this connection.
        schedule_on_instance (Union[Unset, str]): Limits running scheduled checks (started by a CRON job scheduler) to
            run only on a named DQOps instance. When this field is empty, data quality checks are run on all DQOps
            instances. Set a DQOps instance name to run checks on a named instance only. The default name of the DQOps Cloud
            SaaS instance is "cloud".
        provider_type (Union[Unset, ProviderType]):
        bigquery (Union[Unset, BigQueryParametersSpec]):
        snowflake (Union[Unset, SnowflakeParametersSpec]):
        postgresql (Union[Unset, PostgresqlParametersSpec]):
        duckdb (Union[Unset, DuckdbParametersSpec]):
        redshift (Union[Unset, RedshiftParametersSpec]):
        sqlserver (Union[Unset, SqlServerParametersSpec]):
        presto (Union[Unset, PrestoParametersSpec]):
        trino (Union[Unset, TrinoParametersSpec]):
        mysql (Union[Unset, MysqlParametersSpec]):
        oracle (Union[Unset, OracleParametersSpec]):
        spark (Union[Unset, SparkParametersSpec]):
        databricks (Union[Unset, DatabricksParametersSpec]):
        hana (Union[Unset, HanaParametersSpec]):
        db2 (Union[Unset, Db2ParametersSpec]):
        mariadb (Union[Unset, MariaDbParametersSpec]):
        clickhouse (Union[Unset, ClickHouseParametersSpec]):
        questdb (Union[Unset, QuestDbParametersSpec]):
        teradata (Union[Unset, TeradataParametersSpec]):
        run_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter, identifies which
            checks on which tables and columns should be executed.
        run_profiling_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        run_monitoring_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        run_partition_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter,
            identifies which checks on which tables and columns should be executed.
        collect_statistics_job_template (Union[Unset, StatisticsCollectorSearchFilters]):
        data_clean_job_template (Union[Unset, DeleteStoredDataQueueJobParameters]):
        advanced_properties (Union[Unset, ConnectionModelAdvancedProperties]): A dictionary of advanced properties that
            can be used for e.g. to support mapping data to data catalogs, a key/value dictionary.
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can update or delete the connection
            to the data source.
        can_collect_statistics (Union[Unset, bool]): Boolean flag that decides if the current user can collect
            statistics.
        can_run_checks (Union[Unset, bool]): Boolean flag that decides if the current user can run checks.
        can_delete_data (Union[Unset, bool]): Boolean flag that decides if the current user can delete data (results).
        yaml_parsing_error (Union[Unset, str]): Optional parsing error that was captured when parsing the YAML file.
            This field is null when the YAML file is valid. If an error was captured, this field returns the file parsing
            error message and the file location.
    """

    connection_name: Union[Unset, str] = UNSET
    connection_hash: Union[Unset, int] = UNSET
    parallel_jobs_limit: Union[Unset, int] = UNSET
    schedule_on_instance: Union[Unset, str] = UNSET
    provider_type: Union[Unset, ProviderType] = UNSET
    bigquery: Union[Unset, "BigQueryParametersSpec"] = UNSET
    snowflake: Union[Unset, "SnowflakeParametersSpec"] = UNSET
    postgresql: Union[Unset, "PostgresqlParametersSpec"] = UNSET
    duckdb: Union[Unset, "DuckdbParametersSpec"] = UNSET
    redshift: Union[Unset, "RedshiftParametersSpec"] = UNSET
    sqlserver: Union[Unset, "SqlServerParametersSpec"] = UNSET
    presto: Union[Unset, "PrestoParametersSpec"] = UNSET
    trino: Union[Unset, "TrinoParametersSpec"] = UNSET
    mysql: Union[Unset, "MysqlParametersSpec"] = UNSET
    oracle: Union[Unset, "OracleParametersSpec"] = UNSET
    spark: Union[Unset, "SparkParametersSpec"] = UNSET
    databricks: Union[Unset, "DatabricksParametersSpec"] = UNSET
    hana: Union[Unset, "HanaParametersSpec"] = UNSET
    db2: Union[Unset, "Db2ParametersSpec"] = UNSET
    mariadb: Union[Unset, "MariaDbParametersSpec"] = UNSET
    clickhouse: Union[Unset, "ClickHouseParametersSpec"] = UNSET
    questdb: Union[Unset, "QuestDbParametersSpec"] = UNSET
    teradata: Union[Unset, "TeradataParametersSpec"] = UNSET
    run_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_profiling_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_monitoring_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    run_partition_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    collect_statistics_job_template: Union[
        Unset, "StatisticsCollectorSearchFilters"
    ] = UNSET
    data_clean_job_template: Union[Unset, "DeleteStoredDataQueueJobParameters"] = UNSET
    advanced_properties: Union[Unset, "ConnectionModelAdvancedProperties"] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    can_collect_statistics: Union[Unset, bool] = UNSET
    can_run_checks: Union[Unset, bool] = UNSET
    can_delete_data: Union[Unset, bool] = UNSET
    yaml_parsing_error: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_name = self.connection_name
        connection_hash = self.connection_hash
        parallel_jobs_limit = self.parallel_jobs_limit
        schedule_on_instance = self.schedule_on_instance
        provider_type: Union[Unset, str] = UNSET
        if not isinstance(self.provider_type, Unset):
            provider_type = self.provider_type.value

        bigquery: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.bigquery, Unset):
            bigquery = self.bigquery.to_dict()

        snowflake: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.snowflake, Unset):
            snowflake = self.snowflake.to_dict()

        postgresql: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.postgresql, Unset):
            postgresql = self.postgresql.to_dict()

        duckdb: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.duckdb, Unset):
            duckdb = self.duckdb.to_dict()

        redshift: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.redshift, Unset):
            redshift = self.redshift.to_dict()

        sqlserver: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sqlserver, Unset):
            sqlserver = self.sqlserver.to_dict()

        presto: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.presto, Unset):
            presto = self.presto.to_dict()

        trino: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.trino, Unset):
            trino = self.trino.to_dict()

        mysql: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mysql, Unset):
            mysql = self.mysql.to_dict()

        oracle: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.oracle, Unset):
            oracle = self.oracle.to_dict()

        spark: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.spark, Unset):
            spark = self.spark.to_dict()

        databricks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.databricks, Unset):
            databricks = self.databricks.to_dict()

        hana: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.hana, Unset):
            hana = self.hana.to_dict()

        db2: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.db2, Unset):
            db2 = self.db2.to_dict()

        mariadb: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.mariadb, Unset):
            mariadb = self.mariadb.to_dict()

        clickhouse: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.clickhouse, Unset):
            clickhouse = self.clickhouse.to_dict()

        questdb: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.questdb, Unset):
            questdb = self.questdb.to_dict()

        teradata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.teradata, Unset):
            teradata = self.teradata.to_dict()

        run_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_checks_job_template, Unset):
            run_checks_job_template = self.run_checks_job_template.to_dict()

        run_profiling_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_profiling_checks_job_template, Unset):
            run_profiling_checks_job_template = (
                self.run_profiling_checks_job_template.to_dict()
            )

        run_monitoring_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_monitoring_checks_job_template, Unset):
            run_monitoring_checks_job_template = (
                self.run_monitoring_checks_job_template.to_dict()
            )

        run_partition_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_partition_checks_job_template, Unset):
            run_partition_checks_job_template = (
                self.run_partition_checks_job_template.to_dict()
            )

        collect_statistics_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.collect_statistics_job_template, Unset):
            collect_statistics_job_template = (
                self.collect_statistics_job_template.to_dict()
            )

        data_clean_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_clean_job_template, Unset):
            data_clean_job_template = self.data_clean_job_template.to_dict()

        advanced_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.advanced_properties, Unset):
            advanced_properties = self.advanced_properties.to_dict()

        can_edit = self.can_edit
        can_collect_statistics = self.can_collect_statistics
        can_run_checks = self.can_run_checks
        can_delete_data = self.can_delete_data
        yaml_parsing_error = self.yaml_parsing_error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_name is not UNSET:
            field_dict["connection_name"] = connection_name
        if connection_hash is not UNSET:
            field_dict["connection_hash"] = connection_hash
        if parallel_jobs_limit is not UNSET:
            field_dict["parallel_jobs_limit"] = parallel_jobs_limit
        if schedule_on_instance is not UNSET:
            field_dict["schedule_on_instance"] = schedule_on_instance
        if provider_type is not UNSET:
            field_dict["provider_type"] = provider_type
        if bigquery is not UNSET:
            field_dict["bigquery"] = bigquery
        if snowflake is not UNSET:
            field_dict["snowflake"] = snowflake
        if postgresql is not UNSET:
            field_dict["postgresql"] = postgresql
        if duckdb is not UNSET:
            field_dict["duckdb"] = duckdb
        if redshift is not UNSET:
            field_dict["redshift"] = redshift
        if sqlserver is not UNSET:
            field_dict["sqlserver"] = sqlserver
        if presto is not UNSET:
            field_dict["presto"] = presto
        if trino is not UNSET:
            field_dict["trino"] = trino
        if mysql is not UNSET:
            field_dict["mysql"] = mysql
        if oracle is not UNSET:
            field_dict["oracle"] = oracle
        if spark is not UNSET:
            field_dict["spark"] = spark
        if databricks is not UNSET:
            field_dict["databricks"] = databricks
        if hana is not UNSET:
            field_dict["hana"] = hana
        if db2 is not UNSET:
            field_dict["db2"] = db2
        if mariadb is not UNSET:
            field_dict["mariadb"] = mariadb
        if clickhouse is not UNSET:
            field_dict["clickhouse"] = clickhouse
        if questdb is not UNSET:
            field_dict["questdb"] = questdb
        if teradata is not UNSET:
            field_dict["teradata"] = teradata
        if run_checks_job_template is not UNSET:
            field_dict["run_checks_job_template"] = run_checks_job_template
        if run_profiling_checks_job_template is not UNSET:
            field_dict["run_profiling_checks_job_template"] = (
                run_profiling_checks_job_template
            )
        if run_monitoring_checks_job_template is not UNSET:
            field_dict["run_monitoring_checks_job_template"] = (
                run_monitoring_checks_job_template
            )
        if run_partition_checks_job_template is not UNSET:
            field_dict["run_partition_checks_job_template"] = (
                run_partition_checks_job_template
            )
        if collect_statistics_job_template is not UNSET:
            field_dict["collect_statistics_job_template"] = (
                collect_statistics_job_template
            )
        if data_clean_job_template is not UNSET:
            field_dict["data_clean_job_template"] = data_clean_job_template
        if advanced_properties is not UNSET:
            field_dict["advanced_properties"] = advanced_properties
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if can_collect_statistics is not UNSET:
            field_dict["can_collect_statistics"] = can_collect_statistics
        if can_run_checks is not UNSET:
            field_dict["can_run_checks"] = can_run_checks
        if can_delete_data is not UNSET:
            field_dict["can_delete_data"] = can_delete_data
        if yaml_parsing_error is not UNSET:
            field_dict["yaml_parsing_error"] = yaml_parsing_error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.big_query_parameters_spec import BigQueryParametersSpec
        from ..models.check_search_filters import CheckSearchFilters
        from ..models.click_house_parameters_spec import ClickHouseParametersSpec
        from ..models.connection_model_advanced_properties import (
            ConnectionModelAdvancedProperties,
        )
        from ..models.databricks_parameters_spec import DatabricksParametersSpec
        from ..models.db_2_parameters_spec import Db2ParametersSpec
        from ..models.delete_stored_data_queue_job_parameters import (
            DeleteStoredDataQueueJobParameters,
        )
        from ..models.duckdb_parameters_spec import DuckdbParametersSpec
        from ..models.hana_parameters_spec import HanaParametersSpec
        from ..models.maria_db_parameters_spec import MariaDbParametersSpec
        from ..models.mysql_parameters_spec import MysqlParametersSpec
        from ..models.oracle_parameters_spec import OracleParametersSpec
        from ..models.postgresql_parameters_spec import PostgresqlParametersSpec
        from ..models.presto_parameters_spec import PrestoParametersSpec
        from ..models.quest_db_parameters_spec import QuestDbParametersSpec
        from ..models.redshift_parameters_spec import RedshiftParametersSpec
        from ..models.snowflake_parameters_spec import SnowflakeParametersSpec
        from ..models.spark_parameters_spec import SparkParametersSpec
        from ..models.sql_server_parameters_spec import SqlServerParametersSpec
        from ..models.statistics_collector_search_filters import (
            StatisticsCollectorSearchFilters,
        )
        from ..models.teradata_parameters_spec import TeradataParametersSpec
        from ..models.trino_parameters_spec import TrinoParametersSpec

        d = src_dict.copy()
        connection_name = d.pop("connection_name", UNSET)

        connection_hash = d.pop("connection_hash", UNSET)

        parallel_jobs_limit = d.pop("parallel_jobs_limit", UNSET)

        schedule_on_instance = d.pop("schedule_on_instance", UNSET)

        _provider_type = d.pop("provider_type", UNSET)
        provider_type: Union[Unset, ProviderType]
        if isinstance(_provider_type, Unset):
            provider_type = UNSET
        else:
            provider_type = ProviderType(_provider_type)

        _bigquery = d.pop("bigquery", UNSET)
        bigquery: Union[Unset, BigQueryParametersSpec]
        if isinstance(_bigquery, Unset):
            bigquery = UNSET
        else:
            bigquery = BigQueryParametersSpec.from_dict(_bigquery)

        _snowflake = d.pop("snowflake", UNSET)
        snowflake: Union[Unset, SnowflakeParametersSpec]
        if isinstance(_snowflake, Unset):
            snowflake = UNSET
        else:
            snowflake = SnowflakeParametersSpec.from_dict(_snowflake)

        _postgresql = d.pop("postgresql", UNSET)
        postgresql: Union[Unset, PostgresqlParametersSpec]
        if isinstance(_postgresql, Unset):
            postgresql = UNSET
        else:
            postgresql = PostgresqlParametersSpec.from_dict(_postgresql)

        _duckdb = d.pop("duckdb", UNSET)
        duckdb: Union[Unset, DuckdbParametersSpec]
        if isinstance(_duckdb, Unset):
            duckdb = UNSET
        else:
            duckdb = DuckdbParametersSpec.from_dict(_duckdb)

        _redshift = d.pop("redshift", UNSET)
        redshift: Union[Unset, RedshiftParametersSpec]
        if isinstance(_redshift, Unset):
            redshift = UNSET
        else:
            redshift = RedshiftParametersSpec.from_dict(_redshift)

        _sqlserver = d.pop("sqlserver", UNSET)
        sqlserver: Union[Unset, SqlServerParametersSpec]
        if isinstance(_sqlserver, Unset):
            sqlserver = UNSET
        else:
            sqlserver = SqlServerParametersSpec.from_dict(_sqlserver)

        _presto = d.pop("presto", UNSET)
        presto: Union[Unset, PrestoParametersSpec]
        if isinstance(_presto, Unset):
            presto = UNSET
        else:
            presto = PrestoParametersSpec.from_dict(_presto)

        _trino = d.pop("trino", UNSET)
        trino: Union[Unset, TrinoParametersSpec]
        if isinstance(_trino, Unset):
            trino = UNSET
        else:
            trino = TrinoParametersSpec.from_dict(_trino)

        _mysql = d.pop("mysql", UNSET)
        mysql: Union[Unset, MysqlParametersSpec]
        if isinstance(_mysql, Unset):
            mysql = UNSET
        else:
            mysql = MysqlParametersSpec.from_dict(_mysql)

        _oracle = d.pop("oracle", UNSET)
        oracle: Union[Unset, OracleParametersSpec]
        if isinstance(_oracle, Unset):
            oracle = UNSET
        else:
            oracle = OracleParametersSpec.from_dict(_oracle)

        _spark = d.pop("spark", UNSET)
        spark: Union[Unset, SparkParametersSpec]
        if isinstance(_spark, Unset):
            spark = UNSET
        else:
            spark = SparkParametersSpec.from_dict(_spark)

        _databricks = d.pop("databricks", UNSET)
        databricks: Union[Unset, DatabricksParametersSpec]
        if isinstance(_databricks, Unset):
            databricks = UNSET
        else:
            databricks = DatabricksParametersSpec.from_dict(_databricks)

        _hana = d.pop("hana", UNSET)
        hana: Union[Unset, HanaParametersSpec]
        if isinstance(_hana, Unset):
            hana = UNSET
        else:
            hana = HanaParametersSpec.from_dict(_hana)

        _db2 = d.pop("db2", UNSET)
        db2: Union[Unset, Db2ParametersSpec]
        if isinstance(_db2, Unset):
            db2 = UNSET
        else:
            db2 = Db2ParametersSpec.from_dict(_db2)

        _mariadb = d.pop("mariadb", UNSET)
        mariadb: Union[Unset, MariaDbParametersSpec]
        if isinstance(_mariadb, Unset):
            mariadb = UNSET
        else:
            mariadb = MariaDbParametersSpec.from_dict(_mariadb)

        _clickhouse = d.pop("clickhouse", UNSET)
        clickhouse: Union[Unset, ClickHouseParametersSpec]
        if isinstance(_clickhouse, Unset):
            clickhouse = UNSET
        else:
            clickhouse = ClickHouseParametersSpec.from_dict(_clickhouse)

        _questdb = d.pop("questdb", UNSET)
        questdb: Union[Unset, QuestDbParametersSpec]
        if isinstance(_questdb, Unset):
            questdb = UNSET
        else:
            questdb = QuestDbParametersSpec.from_dict(_questdb)

        _teradata = d.pop("teradata", UNSET)
        teradata: Union[Unset, TeradataParametersSpec]
        if isinstance(_teradata, Unset):
            teradata = UNSET
        else:
            teradata = TeradataParametersSpec.from_dict(_teradata)

        _run_checks_job_template = d.pop("run_checks_job_template", UNSET)
        run_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_checks_job_template, Unset):
            run_checks_job_template = UNSET
        else:
            run_checks_job_template = CheckSearchFilters.from_dict(
                _run_checks_job_template
            )

        _run_profiling_checks_job_template = d.pop(
            "run_profiling_checks_job_template", UNSET
        )
        run_profiling_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_profiling_checks_job_template, Unset):
            run_profiling_checks_job_template = UNSET
        else:
            run_profiling_checks_job_template = CheckSearchFilters.from_dict(
                _run_profiling_checks_job_template
            )

        _run_monitoring_checks_job_template = d.pop(
            "run_monitoring_checks_job_template", UNSET
        )
        run_monitoring_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_monitoring_checks_job_template, Unset):
            run_monitoring_checks_job_template = UNSET
        else:
            run_monitoring_checks_job_template = CheckSearchFilters.from_dict(
                _run_monitoring_checks_job_template
            )

        _run_partition_checks_job_template = d.pop(
            "run_partition_checks_job_template", UNSET
        )
        run_partition_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_partition_checks_job_template, Unset):
            run_partition_checks_job_template = UNSET
        else:
            run_partition_checks_job_template = CheckSearchFilters.from_dict(
                _run_partition_checks_job_template
            )

        _collect_statistics_job_template = d.pop(
            "collect_statistics_job_template", UNSET
        )
        collect_statistics_job_template: Union[Unset, StatisticsCollectorSearchFilters]
        if isinstance(_collect_statistics_job_template, Unset):
            collect_statistics_job_template = UNSET
        else:
            collect_statistics_job_template = (
                StatisticsCollectorSearchFilters.from_dict(
                    _collect_statistics_job_template
                )
            )

        _data_clean_job_template = d.pop("data_clean_job_template", UNSET)
        data_clean_job_template: Union[Unset, DeleteStoredDataQueueJobParameters]
        if isinstance(_data_clean_job_template, Unset):
            data_clean_job_template = UNSET
        else:
            data_clean_job_template = DeleteStoredDataQueueJobParameters.from_dict(
                _data_clean_job_template
            )

        _advanced_properties = d.pop("advanced_properties", UNSET)
        advanced_properties: Union[Unset, ConnectionModelAdvancedProperties]
        if isinstance(_advanced_properties, Unset):
            advanced_properties = UNSET
        else:
            advanced_properties = ConnectionModelAdvancedProperties.from_dict(
                _advanced_properties
            )

        can_edit = d.pop("can_edit", UNSET)

        can_collect_statistics = d.pop("can_collect_statistics", UNSET)

        can_run_checks = d.pop("can_run_checks", UNSET)

        can_delete_data = d.pop("can_delete_data", UNSET)

        yaml_parsing_error = d.pop("yaml_parsing_error", UNSET)

        connection_model = cls(
            connection_name=connection_name,
            connection_hash=connection_hash,
            parallel_jobs_limit=parallel_jobs_limit,
            schedule_on_instance=schedule_on_instance,
            provider_type=provider_type,
            bigquery=bigquery,
            snowflake=snowflake,
            postgresql=postgresql,
            duckdb=duckdb,
            redshift=redshift,
            sqlserver=sqlserver,
            presto=presto,
            trino=trino,
            mysql=mysql,
            oracle=oracle,
            spark=spark,
            databricks=databricks,
            hana=hana,
            db2=db2,
            mariadb=mariadb,
            clickhouse=clickhouse,
            questdb=questdb,
            teradata=teradata,
            run_checks_job_template=run_checks_job_template,
            run_profiling_checks_job_template=run_profiling_checks_job_template,
            run_monitoring_checks_job_template=run_monitoring_checks_job_template,
            run_partition_checks_job_template=run_partition_checks_job_template,
            collect_statistics_job_template=collect_statistics_job_template,
            data_clean_job_template=data_clean_job_template,
            advanced_properties=advanced_properties,
            can_edit=can_edit,
            can_collect_statistics=can_collect_statistics,
            can_run_checks=can_run_checks,
            can_delete_data=can_delete_data,
            yaml_parsing_error=yaml_parsing_error,
        )

        connection_model.additional_properties = d
        return connection_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.cloud_synchronization_folders_status_model import (
        CloudSynchronizationFoldersStatusModel,
    )
    from ..models.dqo_job_history_entry_model import DqoJobHistoryEntryModel


T = TypeVar("T", bound="DqoJobQueueInitialSnapshotModel")


@_attrs_define
class DqoJobQueueInitialSnapshotModel:
    """
    Attributes:
        jobs (Union[Unset, List['DqoJobHistoryEntryModel']]):
        folder_synchronization_status (Union[Unset, CloudSynchronizationFoldersStatusModel]):
        last_sequence_number (Union[Unset, int]):
    """

    jobs: Union[Unset, List["DqoJobHistoryEntryModel"]] = UNSET
    folder_synchronization_status: Union[
        Unset, "CloudSynchronizationFoldersStatusModel"
    ] = UNSET
    last_sequence_number: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        jobs: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.jobs, Unset):
            jobs = []
            for jobs_item_data in self.jobs:
                jobs_item = jobs_item_data.to_dict()

                jobs.append(jobs_item)

        folder_synchronization_status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.folder_synchronization_status, Unset):
            folder_synchronization_status = self.folder_synchronization_status.to_dict()

        last_sequence_number = self.last_sequence_number

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if jobs is not UNSET:
            field_dict["jobs"] = jobs
        if folder_synchronization_status is not UNSET:
            field_dict["folderSynchronizationStatus"] = folder_synchronization_status
        if last_sequence_number is not UNSET:
            field_dict["lastSequenceNumber"] = last_sequence_number

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.cloud_synchronization_folders_status_model import (
            CloudSynchronizationFoldersStatusModel,
        )
        from ..models.dqo_job_history_entry_model import DqoJobHistoryEntryModel

        d = src_dict.copy()
        jobs = []
        _jobs = d.pop("jobs", UNSET)
        for jobs_item_data in _jobs or []:
            jobs_item = DqoJobHistoryEntryModel.from_dict(jobs_item_data)

            jobs.append(jobs_item)

        _folder_synchronization_status = d.pop("folderSynchronizationStatus", UNSET)
        folder_synchronization_status: Union[
            Unset, CloudSynchronizationFoldersStatusModel
        ]
        if isinstance(_folder_synchronization_status, Unset):
            folder_synchronization_status = UNSET
        else:
            folder_synchronization_status = (
                CloudSynchronizationFoldersStatusModel.from_dict(
                    _folder_synchronization_status
                )
            )

        last_sequence_number = d.pop("lastSequenceNumber", UNSET)

        dqo_job_queue_initial_snapshot_model = cls(
            jobs=jobs,
            folder_synchronization_status=folder_synchronization_status,
            last_sequence_number=last_sequence_number,
        )

        dqo_job_queue_initial_snapshot_model.additional_properties = d
        return dqo_job_queue_initial_snapshot_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_empty_column_found_check_spec import (
        ColumnEmptyColumnFoundCheckSpec,
    )
    from ..models.column_not_nulls_count_check_spec import ColumnNotNullsCountCheckSpec
    from ..models.column_not_nulls_percent_check_spec import (
        ColumnNotNullsPercentCheckSpec,
    )
    from ..models.column_nulls_count_check_spec import ColumnNullsCountCheckSpec
    from ..models.column_nulls_monthly_monitoring_checks_spec_custom_checks import (
        ColumnNullsMonthlyMonitoringChecksSpecCustomChecks,
    )
    from ..models.column_nulls_percent_check_spec import ColumnNullsPercentCheckSpec


T = TypeVar("T", bound="ColumnNullsMonthlyMonitoringChecksSpec")


@_attrs_define
class ColumnNullsMonthlyMonitoringChecksSpec:
    """
    Attributes:
        custom_checks (Union[Unset, ColumnNullsMonthlyMonitoringChecksSpecCustomChecks]): Dictionary of additional
            custom checks within this category. The keys are check names defined in the definition section. The sensor
            parameters and rules should match the type of the configured sensor and rule for the custom check.
        monthly_nulls_count (Union[Unset, ColumnNullsCountCheckSpec]):
        monthly_nulls_percent (Union[Unset, ColumnNullsPercentCheckSpec]):
        monthly_not_nulls_count (Union[Unset, ColumnNotNullsCountCheckSpec]):
        monthly_not_nulls_percent (Union[Unset, ColumnNotNullsPercentCheckSpec]):
        monthly_empty_column_found (Union[Unset, ColumnEmptyColumnFoundCheckSpec]):
    """

    custom_checks: Union[
        Unset, "ColumnNullsMonthlyMonitoringChecksSpecCustomChecks"
    ] = UNSET
    monthly_nulls_count: Union[Unset, "ColumnNullsCountCheckSpec"] = UNSET
    monthly_nulls_percent: Union[Unset, "ColumnNullsPercentCheckSpec"] = UNSET
    monthly_not_nulls_count: Union[Unset, "ColumnNotNullsCountCheckSpec"] = UNSET
    monthly_not_nulls_percent: Union[Unset, "ColumnNotNullsPercentCheckSpec"] = UNSET
    monthly_empty_column_found: Union[Unset, "ColumnEmptyColumnFoundCheckSpec"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_checks, Unset):
            custom_checks = self.custom_checks.to_dict()

        monthly_nulls_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_nulls_count, Unset):
            monthly_nulls_count = self.monthly_nulls_count.to_dict()

        monthly_nulls_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_nulls_percent, Unset):
            monthly_nulls_percent = self.monthly_nulls_percent.to_dict()

        monthly_not_nulls_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_not_nulls_count, Unset):
            monthly_not_nulls_count = self.monthly_not_nulls_count.to_dict()

        monthly_not_nulls_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_not_nulls_percent, Unset):
            monthly_not_nulls_percent = self.monthly_not_nulls_percent.to_dict()

        monthly_empty_column_found: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.monthly_empty_column_found, Unset):
            monthly_empty_column_found = self.monthly_empty_column_found.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom_checks is not UNSET:
            field_dict["custom_checks"] = custom_checks
        if monthly_nulls_count is not UNSET:
            field_dict["monthly_nulls_count"] = monthly_nulls_count
        if monthly_nulls_percent is not UNSET:
            field_dict["monthly_nulls_percent"] = monthly_nulls_percent
        if monthly_not_nulls_count is not UNSET:
            field_dict["monthly_not_nulls_count"] = monthly_not_nulls_count
        if monthly_not_nulls_percent is not UNSET:
            field_dict["monthly_not_nulls_percent"] = monthly_not_nulls_percent
        if monthly_empty_column_found is not UNSET:
            field_dict["monthly_empty_column_found"] = monthly_empty_column_found

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_empty_column_found_check_spec import (
            ColumnEmptyColumnFoundCheckSpec,
        )
        from ..models.column_not_nulls_count_check_spec import (
            ColumnNotNullsCountCheckSpec,
        )
        from ..models.column_not_nulls_percent_check_spec import (
            ColumnNotNullsPercentCheckSpec,
        )
        from ..models.column_nulls_count_check_spec import ColumnNullsCountCheckSpec
        from ..models.column_nulls_monthly_monitoring_checks_spec_custom_checks import (
            ColumnNullsMonthlyMonitoringChecksSpecCustomChecks,
        )
        from ..models.column_nulls_percent_check_spec import ColumnNullsPercentCheckSpec

        d = src_dict.copy()
        _custom_checks = d.pop("custom_checks", UNSET)
        custom_checks: Union[Unset, ColumnNullsMonthlyMonitoringChecksSpecCustomChecks]
        if isinstance(_custom_checks, Unset):
            custom_checks = UNSET
        else:
            custom_checks = (
                ColumnNullsMonthlyMonitoringChecksSpecCustomChecks.from_dict(
                    _custom_checks
                )
            )

        _monthly_nulls_count = d.pop("monthly_nulls_count", UNSET)
        monthly_nulls_count: Union[Unset, ColumnNullsCountCheckSpec]
        if isinstance(_monthly_nulls_count, Unset):
            monthly_nulls_count = UNSET
        else:
            monthly_nulls_count = ColumnNullsCountCheckSpec.from_dict(
                _monthly_nulls_count
            )

        _monthly_nulls_percent = d.pop("monthly_nulls_percent", UNSET)
        monthly_nulls_percent: Union[Unset, ColumnNullsPercentCheckSpec]
        if isinstance(_monthly_nulls_percent, Unset):
            monthly_nulls_percent = UNSET
        else:
            monthly_nulls_percent = ColumnNullsPercentCheckSpec.from_dict(
                _monthly_nulls_percent
            )

        _monthly_not_nulls_count = d.pop("monthly_not_nulls_count", UNSET)
        monthly_not_nulls_count: Union[Unset, ColumnNotNullsCountCheckSpec]
        if isinstance(_monthly_not_nulls_count, Unset):
            monthly_not_nulls_count = UNSET
        else:
            monthly_not_nulls_count = ColumnNotNullsCountCheckSpec.from_dict(
                _monthly_not_nulls_count
            )

        _monthly_not_nulls_percent = d.pop("monthly_not_nulls_percent", UNSET)
        monthly_not_nulls_percent: Union[Unset, ColumnNotNullsPercentCheckSpec]
        if isinstance(_monthly_not_nulls_percent, Unset):
            monthly_not_nulls_percent = UNSET
        else:
            monthly_not_nulls_percent = ColumnNotNullsPercentCheckSpec.from_dict(
                _monthly_not_nulls_percent
            )

        _monthly_empty_column_found = d.pop("monthly_empty_column_found", UNSET)
        monthly_empty_column_found: Union[Unset, ColumnEmptyColumnFoundCheckSpec]
        if isinstance(_monthly_empty_column_found, Unset):
            monthly_empty_column_found = UNSET
        else:
            monthly_empty_column_found = ColumnEmptyColumnFoundCheckSpec.from_dict(
                _monthly_empty_column_found
            )

        column_nulls_monthly_monitoring_checks_spec = cls(
            custom_checks=custom_checks,
            monthly_nulls_count=monthly_nulls_count,
            monthly_nulls_percent=monthly_nulls_percent,
            monthly_not_nulls_count=monthly_not_nulls_count,
            monthly_not_nulls_percent=monthly_not_nulls_percent,
            monthly_empty_column_found=monthly_empty_column_found,
        )

        column_nulls_monthly_monitoring_checks_spec.additional_properties = d
        return column_nulls_monthly_monitoring_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.import_tables_queue_job_parameters import (
        ImportTablesQueueJobParameters,
    )


T = TypeVar("T", bound="SchemaRemoteModel")


@_attrs_define
class SchemaRemoteModel:
    """Schema remote model

    Attributes:
        connection_name (Union[Unset, str]): Connection name.
        schema_name (Union[Unset, str]): Schema name.
        already_imported (Union[Unset, bool]): Has the schema been imported.
        import_table_job_parameters (Union[Unset, ImportTablesQueueJobParameters]):
    """

    connection_name: Union[Unset, str] = UNSET
    schema_name: Union[Unset, str] = UNSET
    already_imported: Union[Unset, bool] = UNSET
    import_table_job_parameters: Union[Unset, "ImportTablesQueueJobParameters"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connection_name = self.connection_name
        schema_name = self.schema_name
        already_imported = self.already_imported
        import_table_job_parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.import_table_job_parameters, Unset):
            import_table_job_parameters = self.import_table_job_parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connection_name is not UNSET:
            field_dict["connectionName"] = connection_name
        if schema_name is not UNSET:
            field_dict["schemaName"] = schema_name
        if already_imported is not UNSET:
            field_dict["alreadyImported"] = already_imported
        if import_table_job_parameters is not UNSET:
            field_dict["importTableJobParameters"] = import_table_job_parameters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.import_tables_queue_job_parameters import (
            ImportTablesQueueJobParameters,
        )

        d = src_dict.copy()
        connection_name = d.pop("connectionName", UNSET)

        schema_name = d.pop("schemaName", UNSET)

        already_imported = d.pop("alreadyImported", UNSET)

        _import_table_job_parameters = d.pop("importTableJobParameters", UNSET)
        import_table_job_parameters: Union[Unset, ImportTablesQueueJobParameters]
        if isinstance(_import_table_job_parameters, Unset):
            import_table_job_parameters = UNSET
        else:
            import_table_job_parameters = ImportTablesQueueJobParameters.from_dict(
                _import_table_job_parameters
            )

        schema_remote_model = cls(
            connection_name=connection_name,
            schema_name=schema_name,
            already_imported=already_imported,
            import_table_job_parameters=import_table_job_parameters,
        )

        schema_remote_model.additional_properties = d
        return schema_remote_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.schedule_enabled_status_model import ScheduleEnabledStatusModel
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.check_search_filters import CheckSearchFilters
    from ..models.delete_stored_data_queue_job_parameters import (
        DeleteStoredDataQueueJobParameters,
    )
    from ..models.effective_schedule_model import EffectiveScheduleModel
    from ..models.quality_category_model import QualityCategoryModel


T = TypeVar("T", bound="CheckContainerModel")


@_attrs_define
class CheckContainerModel:
    """Model that returns the form definition and the form data to edit all data quality checks divided by categories.

    Attributes:
        categories (Union[Unset, List['QualityCategoryModel']]): List of all data quality categories that contain data
            quality checks inside.
        effective_schedule (Union[Unset, EffectiveScheduleModel]): Model of a configured schedule (enabled on connection
            or table) or schedule override (on check). Describes the CRON expression and the time of the upcoming execution,
            as well as the duration until this time.
        effective_schedule_enabled_status (Union[Unset, ScheduleEnabledStatusModel]):
        partition_by_column (Union[Unset, str]): The name of the column that partitioned checks will use for the time
            period partitioning. Important only for partitioned checks.
        run_checks_job_template (Union[Unset, CheckSearchFilters]): Target data quality checks filter, identifies which
            checks on which tables and columns should be executed.
        data_clean_job_template (Union[Unset, DeleteStoredDataQueueJobParameters]):
        can_edit (Union[Unset, bool]): Boolean flag that decides if the current user can edit the check.
        can_run_checks (Union[Unset, bool]): Boolean flag that decides if the current user can run checks.
        can_delete_data (Union[Unset, bool]): Boolean flag that decides if the current user can delete data (results).
    """

    categories: Union[Unset, List["QualityCategoryModel"]] = UNSET
    effective_schedule: Union[Unset, "EffectiveScheduleModel"] = UNSET
    effective_schedule_enabled_status: Union[Unset, ScheduleEnabledStatusModel] = UNSET
    partition_by_column: Union[Unset, str] = UNSET
    run_checks_job_template: Union[Unset, "CheckSearchFilters"] = UNSET
    data_clean_job_template: Union[Unset, "DeleteStoredDataQueueJobParameters"] = UNSET
    can_edit: Union[Unset, bool] = UNSET
    can_run_checks: Union[Unset, bool] = UNSET
    can_delete_data: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        categories: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.categories, Unset):
            categories = []
            for categories_item_data in self.categories:
                categories_item = categories_item_data.to_dict()

                categories.append(categories_item)

        effective_schedule: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.effective_schedule, Unset):
            effective_schedule = self.effective_schedule.to_dict()

        effective_schedule_enabled_status: Union[Unset, str] = UNSET
        if not isinstance(self.effective_schedule_enabled_status, Unset):
            effective_schedule_enabled_status = (
                self.effective_schedule_enabled_status.value
            )

        partition_by_column = self.partition_by_column
        run_checks_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.run_checks_job_template, Unset):
            run_checks_job_template = self.run_checks_job_template.to_dict()

        data_clean_job_template: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.data_clean_job_template, Unset):
            data_clean_job_template = self.data_clean_job_template.to_dict()

        can_edit = self.can_edit
        can_run_checks = self.can_run_checks
        can_delete_data = self.can_delete_data

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if categories is not UNSET:
            field_dict["categories"] = categories
        if effective_schedule is not UNSET:
            field_dict["effective_schedule"] = effective_schedule
        if effective_schedule_enabled_status is not UNSET:
            field_dict["effective_schedule_enabled_status"] = (
                effective_schedule_enabled_status
            )
        if partition_by_column is not UNSET:
            field_dict["partition_by_column"] = partition_by_column
        if run_checks_job_template is not UNSET:
            field_dict["run_checks_job_template"] = run_checks_job_template
        if data_clean_job_template is not UNSET:
            field_dict["data_clean_job_template"] = data_clean_job_template
        if can_edit is not UNSET:
            field_dict["can_edit"] = can_edit
        if can_run_checks is not UNSET:
            field_dict["can_run_checks"] = can_run_checks
        if can_delete_data is not UNSET:
            field_dict["can_delete_data"] = can_delete_data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.check_search_filters import CheckSearchFilters
        from ..models.delete_stored_data_queue_job_parameters import (
            DeleteStoredDataQueueJobParameters,
        )
        from ..models.effective_schedule_model import EffectiveScheduleModel
        from ..models.quality_category_model import QualityCategoryModel

        d = src_dict.copy()
        categories = []
        _categories = d.pop("categories", UNSET)
        for categories_item_data in _categories or []:
            categories_item = QualityCategoryModel.from_dict(categories_item_data)

            categories.append(categories_item)

        _effective_schedule = d.pop("effective_schedule", UNSET)
        effective_schedule: Union[Unset, EffectiveScheduleModel]
        if isinstance(_effective_schedule, Unset):
            effective_schedule = UNSET
        else:
            effective_schedule = EffectiveScheduleModel.from_dict(_effective_schedule)

        _effective_schedule_enabled_status = d.pop(
            "effective_schedule_enabled_status", UNSET
        )
        effective_schedule_enabled_status: Union[Unset, ScheduleEnabledStatusModel]
        if isinstance(_effective_schedule_enabled_status, Unset):
            effective_schedule_enabled_status = UNSET
        else:
            effective_schedule_enabled_status = ScheduleEnabledStatusModel(
                _effective_schedule_enabled_status
            )

        partition_by_column = d.pop("partition_by_column", UNSET)

        _run_checks_job_template = d.pop("run_checks_job_template", UNSET)
        run_checks_job_template: Union[Unset, CheckSearchFilters]
        if isinstance(_run_checks_job_template, Unset):
            run_checks_job_template = UNSET
        else:
            run_checks_job_template = CheckSearchFilters.from_dict(
                _run_checks_job_template
            )

        _data_clean_job_template = d.pop("data_clean_job_template", UNSET)
        data_clean_job_template: Union[Unset, DeleteStoredDataQueueJobParameters]
        if isinstance(_data_clean_job_template, Unset):
            data_clean_job_template = UNSET
        else:
            data_clean_job_template = DeleteStoredDataQueueJobParameters.from_dict(
                _data_clean_job_template
            )

        can_edit = d.pop("can_edit", UNSET)

        can_run_checks = d.pop("can_run_checks", UNSET)

        can_delete_data = d.pop("can_delete_data", UNSET)

        check_container_model = cls(
            categories=categories,
            effective_schedule=effective_schedule,
            effective_schedule_enabled_status=effective_schedule_enabled_status,
            partition_by_column=partition_by_column,
            run_checks_job_template=run_checks_job_template,
            data_clean_job_template=data_clean_job_template,
            can_edit=can_edit,
            can_run_checks=can_run_checks,
            can_delete_data=can_delete_data,
        )

        check_container_model.additional_properties = d
        return check_container_model

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DqoUserRole(str, Enum):
    ADMIN = "admin"
    EDITOR = "editor"
    NONE = "none"
    OPERATOR = "operator"
    VIEWER = "viewer"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.custom_check_spec import CustomCheckSpec


T = TypeVar("T", bound="TableComparisonMonthlyPartitionedChecksSpecCustomChecks")


@_attrs_define
class TableComparisonMonthlyPartitionedChecksSpecCustomChecks:
    """Dictionary of additional custom checks within this category. The keys are check names defined in the definition
    section. The sensor parameters and rules should match the type of the configured sensor and rule for the custom
    check.

    """

    additional_properties: Dict[str, "CustomCheckSpec"] = _attrs_field(
        init=False, factory=dict
    )

    def to_dict(self) -> Dict[str, Any]:
        pass

        field_dict: Dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = prop.to_dict()

        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.custom_check_spec import CustomCheckSpec

        d = src_dict.copy()
        table_comparison_monthly_partitioned_checks_spec_custom_checks = cls()

        additional_properties = {}
        for prop_name, prop_dict in d.items():
            additional_property = CustomCheckSpec.from_dict(prop_dict)

            additional_properties[prop_name] = additional_property

        table_comparison_monthly_partitioned_checks_spec_custom_checks.additional_properties = (
            additional_properties
        )
        return table_comparison_monthly_partitioned_checks_spec_custom_checks

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> "CustomCheckSpec":
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: "CustomCheckSpec") -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.column_empty_column_found_check_spec import (
        ColumnEmptyColumnFoundCheckSpec,
    )
    from ..models.column_not_nulls_count_check_spec import ColumnNotNullsCountCheckSpec
    from ..models.column_not_nulls_percent_check_spec import (
        ColumnNotNullsPercentCheckSpec,
    )
    from ..models.column_null_percent_anomaly_stationary_check_spec import (
        ColumnNullPercentAnomalyStationaryCheckSpec,
    )
    from ..models.column_null_percent_change_1_day_check_spec import (
        ColumnNullPercentChange1DayCheckSpec,
    )
    from ..models.column_null_percent_change_7_days_check_spec import (
        ColumnNullPercentChange7DaysCheckSpec,
    )
    from ..models.column_null_percent_change_30_days_check_spec import (
        ColumnNullPercentChange30DaysCheckSpec,
    )
    from ..models.column_null_percent_change_check_spec import (
        ColumnNullPercentChangeCheckSpec,
    )
    from ..models.column_nulls_count_check_spec import ColumnNullsCountCheckSpec
    from ..models.column_nulls_daily_monitoring_checks_spec_custom_checks import (
        ColumnNullsDailyMonitoringChecksSpecCustomChecks,
    )
    from ..models.column_nulls_percent_check_spec import ColumnNullsPercentCheckSpec


T = TypeVar("T", bound="ColumnNullsDailyMonitoringChecksSpec")


@_attrs_define
class ColumnNullsDailyMonitoringChecksSpec:
    """
    Attributes:
        custom_checks (Union[Unset, ColumnNullsDailyMonitoringChecksSpecCustomChecks]): Dictionary of additional custom
            checks within this category. The keys are check names defined in the definition section. The sensor parameters
            and rules should match the type of the configured sensor and rule for the custom check.
        daily_nulls_count (Union[Unset, ColumnNullsCountCheckSpec]):
        daily_nulls_percent (Union[Unset, ColumnNullsPercentCheckSpec]):
        daily_nulls_percent_anomaly (Union[Unset, ColumnNullPercentAnomalyStationaryCheckSpec]):
        daily_not_nulls_count (Union[Unset, ColumnNotNullsCountCheckSpec]):
        daily_not_nulls_percent (Union[Unset, ColumnNotNullsPercentCheckSpec]):
        daily_empty_column_found (Union[Unset, ColumnEmptyColumnFoundCheckSpec]):
        daily_nulls_percent_change (Union[Unset, ColumnNullPercentChangeCheckSpec]):
        daily_nulls_percent_change_1_day (Union[Unset, ColumnNullPercentChange1DayCheckSpec]):
        daily_nulls_percent_change_7_days (Union[Unset, ColumnNullPercentChange7DaysCheckSpec]):
        daily_nulls_percent_change_30_days (Union[Unset, ColumnNullPercentChange30DaysCheckSpec]):
    """

    custom_checks: Union[Unset, "ColumnNullsDailyMonitoringChecksSpecCustomChecks"] = (
        UNSET
    )
    daily_nulls_count: Union[Unset, "ColumnNullsCountCheckSpec"] = UNSET
    daily_nulls_percent: Union[Unset, "ColumnNullsPercentCheckSpec"] = UNSET
    daily_nulls_percent_anomaly: Union[
        Unset, "ColumnNullPercentAnomalyStationaryCheckSpec"
    ] = UNSET
    daily_not_nulls_count: Union[Unset, "ColumnNotNullsCountCheckSpec"] = UNSET
    daily_not_nulls_percent: Union[Unset, "ColumnNotNullsPercentCheckSpec"] = UNSET
    daily_empty_column_found: Union[Unset, "ColumnEmptyColumnFoundCheckSpec"] = UNSET
    daily_nulls_percent_change: Union[Unset, "ColumnNullPercentChangeCheckSpec"] = UNSET
    daily_nulls_percent_change_1_day: Union[
        Unset, "ColumnNullPercentChange1DayCheckSpec"
    ] = UNSET
    daily_nulls_percent_change_7_days: Union[
        Unset, "ColumnNullPercentChange7DaysCheckSpec"
    ] = UNSET
    daily_nulls_percent_change_30_days: Union[
        Unset, "ColumnNullPercentChange30DaysCheckSpec"
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        custom_checks: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_checks, Unset):
            custom_checks = self.custom_checks.to_dict()

        daily_nulls_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_nulls_count, Unset):
            daily_nulls_count = self.daily_nulls_count.to_dict()

        daily_nulls_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_nulls_percent, Unset):
            daily_nulls_percent = self.daily_nulls_percent.to_dict()

        daily_nulls_percent_anomaly: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_nulls_percent_anomaly, Unset):
            daily_nulls_percent_anomaly = self.daily_nulls_percent_anomaly.to_dict()

        daily_not_nulls_count: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_not_nulls_count, Unset):
            daily_not_nulls_count = self.daily_not_nulls_count.to_dict()

        daily_not_nulls_percent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_not_nulls_percent, Unset):
            daily_not_nulls_percent = self.daily_not_nulls_percent.to_dict()

        daily_empty_column_found: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_empty_column_found, Unset):
            daily_empty_column_found = self.daily_empty_column_found.to_dict()

        daily_nulls_percent_change: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_nulls_percent_change, Unset):
            daily_nulls_percent_change = self.daily_nulls_percent_change.to_dict()

        daily_nulls_percent_change_1_day: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_nulls_percent_change_1_day, Unset):
            daily_nulls_percent_change_1_day = (
                self.daily_nulls_percent_change_1_day.to_dict()
            )

        daily_nulls_percent_change_7_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_nulls_percent_change_7_days, Unset):
            daily_nulls_percent_change_7_days = (
                self.daily_nulls_percent_change_7_days.to_dict()
            )

        daily_nulls_percent_change_30_days: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.daily_nulls_percent_change_30_days, Unset):
            daily_nulls_percent_change_30_days = (
                self.daily_nulls_percent_change_30_days.to_dict()
            )

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if custom_checks is not UNSET:
            field_dict["custom_checks"] = custom_checks
        if daily_nulls_count is not UNSET:
            field_dict["daily_nulls_count"] = daily_nulls_count
        if daily_nulls_percent is not UNSET:
            field_dict["daily_nulls_percent"] = daily_nulls_percent
        if daily_nulls_percent_anomaly is not UNSET:
            field_dict["daily_nulls_percent_anomaly"] = daily_nulls_percent_anomaly
        if daily_not_nulls_count is not UNSET:
            field_dict["daily_not_nulls_count"] = daily_not_nulls_count
        if daily_not_nulls_percent is not UNSET:
            field_dict["daily_not_nulls_percent"] = daily_not_nulls_percent
        if daily_empty_column_found is not UNSET:
            field_dict["daily_empty_column_found"] = daily_empty_column_found
        if daily_nulls_percent_change is not UNSET:
            field_dict["daily_nulls_percent_change"] = daily_nulls_percent_change
        if daily_nulls_percent_change_1_day is not UNSET:
            field_dict["daily_nulls_percent_change_1_day"] = (
                daily_nulls_percent_change_1_day
            )
        if daily_nulls_percent_change_7_days is not UNSET:
            field_dict["daily_nulls_percent_change_7_days"] = (
                daily_nulls_percent_change_7_days
            )
        if daily_nulls_percent_change_30_days is not UNSET:
            field_dict["daily_nulls_percent_change_30_days"] = (
                daily_nulls_percent_change_30_days
            )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.column_empty_column_found_check_spec import (
            ColumnEmptyColumnFoundCheckSpec,
        )
        from ..models.column_not_nulls_count_check_spec import (
            ColumnNotNullsCountCheckSpec,
        )
        from ..models.column_not_nulls_percent_check_spec import (
            ColumnNotNullsPercentCheckSpec,
        )
        from ..models.column_null_percent_anomaly_stationary_check_spec import (
            ColumnNullPercentAnomalyStationaryCheckSpec,
        )
        from ..models.column_null_percent_change_1_day_check_spec import (
            ColumnNullPercentChange1DayCheckSpec,
        )
        from ..models.column_null_percent_change_7_days_check_spec import (
            ColumnNullPercentChange7DaysCheckSpec,
        )
        from ..models.column_null_percent_change_30_days_check_spec import (
            ColumnNullPercentChange30DaysCheckSpec,
        )
        from ..models.column_null_percent_change_check_spec import (
            ColumnNullPercentChangeCheckSpec,
        )
        from ..models.column_nulls_count_check_spec import ColumnNullsCountCheckSpec
        from ..models.column_nulls_daily_monitoring_checks_spec_custom_checks import (
            ColumnNullsDailyMonitoringChecksSpecCustomChecks,
        )
        from ..models.column_nulls_percent_check_spec import ColumnNullsPercentCheckSpec

        d = src_dict.copy()
        _custom_checks = d.pop("custom_checks", UNSET)
        custom_checks: Union[Unset, ColumnNullsDailyMonitoringChecksSpecCustomChecks]
        if isinstance(_custom_checks, Unset):
            custom_checks = UNSET
        else:
            custom_checks = ColumnNullsDailyMonitoringChecksSpecCustomChecks.from_dict(
                _custom_checks
            )

        _daily_nulls_count = d.pop("daily_nulls_count", UNSET)
        daily_nulls_count: Union[Unset, ColumnNullsCountCheckSpec]
        if isinstance(_daily_nulls_count, Unset):
            daily_nulls_count = UNSET
        else:
            daily_nulls_count = ColumnNullsCountCheckSpec.from_dict(_daily_nulls_count)

        _daily_nulls_percent = d.pop("daily_nulls_percent", UNSET)
        daily_nulls_percent: Union[Unset, ColumnNullsPercentCheckSpec]
        if isinstance(_daily_nulls_percent, Unset):
            daily_nulls_percent = UNSET
        else:
            daily_nulls_percent = ColumnNullsPercentCheckSpec.from_dict(
                _daily_nulls_percent
            )

        _daily_nulls_percent_anomaly = d.pop("daily_nulls_percent_anomaly", UNSET)
        daily_nulls_percent_anomaly: Union[
            Unset, ColumnNullPercentAnomalyStationaryCheckSpec
        ]
        if isinstance(_daily_nulls_percent_anomaly, Unset):
            daily_nulls_percent_anomaly = UNSET
        else:
            daily_nulls_percent_anomaly = (
                ColumnNullPercentAnomalyStationaryCheckSpec.from_dict(
                    _daily_nulls_percent_anomaly
                )
            )

        _daily_not_nulls_count = d.pop("daily_not_nulls_count", UNSET)
        daily_not_nulls_count: Union[Unset, ColumnNotNullsCountCheckSpec]
        if isinstance(_daily_not_nulls_count, Unset):
            daily_not_nulls_count = UNSET
        else:
            daily_not_nulls_count = ColumnNotNullsCountCheckSpec.from_dict(
                _daily_not_nulls_count
            )

        _daily_not_nulls_percent = d.pop("daily_not_nulls_percent", UNSET)
        daily_not_nulls_percent: Union[Unset, ColumnNotNullsPercentCheckSpec]
        if isinstance(_daily_not_nulls_percent, Unset):
            daily_not_nulls_percent = UNSET
        else:
            daily_not_nulls_percent = ColumnNotNullsPercentCheckSpec.from_dict(
                _daily_not_nulls_percent
            )

        _daily_empty_column_found = d.pop("daily_empty_column_found", UNSET)
        daily_empty_column_found: Union[Unset, ColumnEmptyColumnFoundCheckSpec]
        if isinstance(_daily_empty_column_found, Unset):
            daily_empty_column_found = UNSET
        else:
            daily_empty_column_found = ColumnEmptyColumnFoundCheckSpec.from_dict(
                _daily_empty_column_found
            )

        _daily_nulls_percent_change = d.pop("daily_nulls_percent_change", UNSET)
        daily_nulls_percent_change: Union[Unset, ColumnNullPercentChangeCheckSpec]
        if isinstance(_daily_nulls_percent_change, Unset):
            daily_nulls_percent_change = UNSET
        else:
            daily_nulls_percent_change = ColumnNullPercentChangeCheckSpec.from_dict(
                _daily_nulls_percent_change
            )

        _daily_nulls_percent_change_1_day = d.pop(
            "daily_nulls_percent_change_1_day", UNSET
        )
        daily_nulls_percent_change_1_day: Union[
            Unset, ColumnNullPercentChange1DayCheckSpec
        ]
        if isinstance(_daily_nulls_percent_change_1_day, Unset):
            daily_nulls_percent_change_1_day = UNSET
        else:
            daily_nulls_percent_change_1_day = (
                ColumnNullPercentChange1DayCheckSpec.from_dict(
                    _daily_nulls_percent_change_1_day
                )
            )

        _daily_nulls_percent_change_7_days = d.pop(
            "daily_nulls_percent_change_7_days", UNSET
        )
        daily_nulls_percent_change_7_days: Union[
            Unset, ColumnNullPercentChange7DaysCheckSpec
        ]
        if isinstance(_daily_nulls_percent_change_7_days, Unset):
            daily_nulls_percent_change_7_days = UNSET
        else:
            daily_nulls_percent_change_7_days = (
                ColumnNullPercentChange7DaysCheckSpec.from_dict(
                    _daily_nulls_percent_change_7_days
                )
            )

        _daily_nulls_percent_change_30_days = d.pop(
            "daily_nulls_percent_change_30_days", UNSET
        )
        daily_nulls_percent_change_30_days: Union[
            Unset, ColumnNullPercentChange30DaysCheckSpec
        ]
        if isinstance(_daily_nulls_percent_change_30_days, Unset):
            daily_nulls_percent_change_30_days = UNSET
        else:
            daily_nulls_percent_change_30_days = (
                ColumnNullPercentChange30DaysCheckSpec.from_dict(
                    _daily_nulls_percent_change_30_days
                )
            )

        column_nulls_daily_monitoring_checks_spec = cls(
            custom_checks=custom_checks,
            daily_nulls_count=daily_nulls_count,
            daily_nulls_percent=daily_nulls_percent,
            daily_nulls_percent_anomaly=daily_nulls_percent_anomaly,
            daily_not_nulls_count=daily_not_nulls_count,
            daily_not_nulls_percent=daily_not_nulls_percent,
            daily_empty_column_found=daily_empty_column_found,
            daily_nulls_percent_change=daily_nulls_percent_change,
            daily_nulls_percent_change_1_day=daily_nulls_percent_change_1_day,
            daily_nulls_percent_change_7_days=daily_nulls_percent_change_7_days,
            daily_nulls_percent_change_30_days=daily_nulls_percent_change_30_days,
        )

        column_nulls_daily_monitoring_checks_spec.additional_properties = d
        return column_nulls_daily_monitoring_checks_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.aws_authentication_mode import AwsAuthenticationMode
from ..models.trino_engine_type import TrinoEngineType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.trino_parameters_spec_properties import TrinoParametersSpecProperties


T = TypeVar("T", bound="TrinoParametersSpec")


@_attrs_define
class TrinoParametersSpec:
    """
    Attributes:
        trino_engine_type (Union[Unset, TrinoEngineType]):
        host (Union[Unset, str]): Trino host name. Supports also a ${TRINO_HOST} configuration with a custom environment
            variable.
        port (Union[Unset, str]): Trino port number. The default port is 8080. Supports also a ${TRINO_PORT}
            configuration with a custom environment variable.
        user (Union[Unset, str]): Trino user name. The value can be in the ${ENVIRONMENT_VARIABLE_NAME} format to use
            dynamic substitution.
        password (Union[Unset, str]): Trino database password. The value can be in the ${ENVIRONMENT_VARIABLE_NAME}
            format to use dynamic substitution.
        aws_authentication_mode (Union[Unset, AwsAuthenticationMode]):
        athena_region (Union[Unset, str]): The AWS Region where queries will be run. Supports also a ${ATHENA_REGION}
            configuration with a custom environment variable.
        catalog (Union[Unset, str]): The catalog that contains the databases and the tables that will be accessed with
            the driver. Supports also a ${TRINO_CATALOG} configuration with a custom environment variable.
        athena_work_group (Union[Unset, str]): The workgroup in which queries will run. Supports also a
            ${ATHENA_WORK_GROUP} configuration with a custom environment variable.
        athena_output_location (Union[Unset, str]): The location in Amazon S3 where query results will be stored.
            Supports also a ${ATHENA_OUTPUT_LOCATION} configuration with a custom environment variable.
        properties (Union[Unset, TrinoParametersSpecProperties]): A dictionary of custom JDBC parameters that are added
            to the JDBC connection string, a key/value dictionary.
        database (Union[Unset, str]):
    """

    trino_engine_type: Union[Unset, TrinoEngineType] = UNSET
    host: Union[Unset, str] = UNSET
    port: Union[Unset, str] = UNSET
    user: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    aws_authentication_mode: Union[Unset, AwsAuthenticationMode] = UNSET
    athena_region: Union[Unset, str] = UNSET
    catalog: Union[Unset, str] = UNSET
    athena_work_group: Union[Unset, str] = UNSET
    athena_output_location: Union[Unset, str] = UNSET
    properties: Union[Unset, "TrinoParametersSpecProperties"] = UNSET
    database: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        trino_engine_type: Union[Unset, str] = UNSET
        if not isinstance(self.trino_engine_type, Unset):
            trino_engine_type = self.trino_engine_type.value

        host = self.host
        port = self.port
        user = self.user
        password = self.password
        aws_authentication_mode: Union[Unset, str] = UNSET
        if not isinstance(self.aws_authentication_mode, Unset):
            aws_authentication_mode = self.aws_authentication_mode.value

        athena_region = self.athena_region
        catalog = self.catalog
        athena_work_group = self.athena_work_group
        athena_output_location = self.athena_output_location
        properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.properties, Unset):
            properties = self.properties.to_dict()

        database = self.database

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if trino_engine_type is not UNSET:
            field_dict["trino_engine_type"] = trino_engine_type
        if host is not UNSET:
            field_dict["host"] = host
        if port is not UNSET:
            field_dict["port"] = port
        if user is not UNSET:
            field_dict["user"] = user
        if password is not UNSET:
            field_dict["password"] = password
        if aws_authentication_mode is not UNSET:
            field_dict["aws_authentication_mode"] = aws_authentication_mode
        if athena_region is not UNSET:
            field_dict["athena_region"] = athena_region
        if catalog is not UNSET:
            field_dict["catalog"] = catalog
        if athena_work_group is not UNSET:
            field_dict["athena_work_group"] = athena_work_group
        if athena_output_location is not UNSET:
            field_dict["athena_output_location"] = athena_output_location
        if properties is not UNSET:
            field_dict["properties"] = properties
        if database is not UNSET:
            field_dict["database"] = database

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.trino_parameters_spec_properties import (
            TrinoParametersSpecProperties,
        )

        d = src_dict.copy()
        _trino_engine_type = d.pop("trino_engine_type", UNSET)
        trino_engine_type: Union[Unset, TrinoEngineType]
        if isinstance(_trino_engine_type, Unset):
            trino_engine_type = UNSET
        else:
            trino_engine_type = TrinoEngineType(_trino_engine_type)

        host = d.pop("host", UNSET)

        port = d.pop("port", UNSET)

        user = d.pop("user", UNSET)

        password = d.pop("password", UNSET)

        _aws_authentication_mode = d.pop("aws_authentication_mode", UNSET)
        aws_authentication_mode: Union[Unset, AwsAuthenticationMode]
        if isinstance(_aws_authentication_mode, Unset):
            aws_authentication_mode = UNSET
        else:
            aws_authentication_mode = AwsAuthenticationMode(_aws_authentication_mode)

        athena_region = d.pop("athena_region", UNSET)

        catalog = d.pop("catalog", UNSET)

        athena_work_group = d.pop("athena_work_group", UNSET)

        athena_output_location = d.pop("athena_output_location", UNSET)

        _properties = d.pop("properties", UNSET)
        properties: Union[Unset, TrinoParametersSpecProperties]
        if isinstance(_properties, Unset):
            properties = UNSET
        else:
            properties = TrinoParametersSpecProperties.from_dict(_properties)

        database = d.pop("database", UNSET)

        trino_parameters_spec = cls(
            trino_engine_type=trino_engine_type,
            host=host,
            port=port,
            user=user,
            password=password,
            aws_authentication_mode=aws_authentication_mode,
            athena_region=athena_region,
            catalog=catalog,
            athena_work_group=athena_work_group,
            athena_output_location=athena_output_location,
            properties=properties,
            database=database,
        )

        trino_parameters_spec.additional_properties = d
        return trino_parameters_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TopIncidentGrouping(str, Enum):
    CATEGORY = "category"
    CONNECTION = "connection"
    DIMENSION = "dimension"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.incident_grouping_level import IncidentGroupingLevel
from ..models.minimum_grouping_severity_level import MinimumGroupingSeverityLevel
from ..types import UNSET, Unset

T = TypeVar("T", bound="TableIncidentGroupingSpec")


@_attrs_define
class TableIncidentGroupingSpec:
    """
    Attributes:
        grouping_level (Union[Unset, IncidentGroupingLevel]):
        minimum_severity (Union[Unset, MinimumGroupingSeverityLevel]):
        divide_by_data_group (Union[Unset, bool]): Create separate data quality incidents for each data group, creating
            different incidents for different groups of rows. By default, data groups are ignored for grouping data quality
            issues into data quality incidents.
        disabled (Union[Unset, bool]): Disables data quality incident creation for failed data quality checks on the
            table.
    """

    grouping_level: Union[Unset, IncidentGroupingLevel] = UNSET
    minimum_severity: Union[Unset, MinimumGroupingSeverityLevel] = UNSET
    divide_by_data_group: Union[Unset, bool] = UNSET
    disabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        grouping_level: Union[Unset, str] = UNSET
        if not isinstance(self.grouping_level, Unset):
            grouping_level = self.grouping_level.value

        minimum_severity: Union[Unset, str] = UNSET
        if not isinstance(self.minimum_severity, Unset):
            minimum_severity = self.minimum_severity.value

        divide_by_data_group = self.divide_by_data_group
        disabled = self.disabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if grouping_level is not UNSET:
            field_dict["grouping_level"] = grouping_level
        if minimum_severity is not UNSET:
            field_dict["minimum_severity"] = minimum_severity
        if divide_by_data_group is not UNSET:
            field_dict["divide_by_data_group"] = divide_by_data_group
        if disabled is not UNSET:
            field_dict["disabled"] = disabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _grouping_level = d.pop("grouping_level", UNSET)
        grouping_level: Union[Unset, IncidentGroupingLevel]
        if isinstance(_grouping_level, Unset):
            grouping_level = UNSET
        else:
            grouping_level = IncidentGroupingLevel(_grouping_level)

        _minimum_severity = d.pop("minimum_severity", UNSET)
        minimum_severity: Union[Unset, MinimumGroupingSeverityLevel]
        if isinstance(_minimum_severity, Unset):
            minimum_severity = UNSET
        else:
            minimum_severity = MinimumGroupingSeverityLevel(_minimum_severity)

        divide_by_data_group = d.pop("divide_by_data_group", UNSET)

        disabled = d.pop("disabled", UNSET)

        table_incident_grouping_spec = cls(
            grouping_level=grouping_level,
            minimum_severity=minimum_severity,
            divide_by_data_group=divide_by_data_group,
            disabled=disabled,
        )

        table_incident_grouping_spec.additional_properties = d
        return table_incident_grouping_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
